// REGEX = "^GMT$"
use core::fmt::{Debug, Formatter, Error};
use core::str::FromStr;

use ::timezone_impl::{TimeSpans, FixedTimespanSet, FixedTimespan};

/// TimeZones built at compile time from the tz database
///
/// This implements [`chrono::TimeZone`] so that it may be used in and to
/// construct chrono's DateTime type. See the root module documentation
/// for details.
#[derive(Clone, Copy, PartialEq, Eq, Hash)]
pub enum Tz {
    /// Etc/GMT
    Etc__GMT,
    /// Etc/GMT+0
    Etc__GMTPlus0,
    /// Etc/GMT+1
    Etc__GMTPlus1,
    /// Etc/GMT+10
    Etc__GMTPlus10,
    /// Etc/GMT+11
    Etc__GMTPlus11,
    /// Etc/GMT+12
    Etc__GMTPlus12,
    /// Etc/GMT+2
    Etc__GMTPlus2,
    /// Etc/GMT+3
    Etc__GMTPlus3,
    /// Etc/GMT+4
    Etc__GMTPlus4,
    /// Etc/GMT+5
    Etc__GMTPlus5,
    /// Etc/GMT+6
    Etc__GMTPlus6,
    /// Etc/GMT+7
    Etc__GMTPlus7,
    /// Etc/GMT+8
    Etc__GMTPlus8,
    /// Etc/GMT+9
    Etc__GMTPlus9,
    /// Etc/GMT-0
    Etc__GMTMinus0,
    /// Etc/GMT-1
    Etc__GMTMinus1,
    /// Etc/GMT-10
    Etc__GMTMinus10,
    /// Etc/GMT-11
    Etc__GMTMinus11,
    /// Etc/GMT-12
    Etc__GMTMinus12,
    /// Etc/GMT-13
    Etc__GMTMinus13,
    /// Etc/GMT-14
    Etc__GMTMinus14,
    /// Etc/GMT-2
    Etc__GMTMinus2,
    /// Etc/GMT-3
    Etc__GMTMinus3,
    /// Etc/GMT-4
    Etc__GMTMinus4,
    /// Etc/GMT-5
    Etc__GMTMinus5,
    /// Etc/GMT-6
    Etc__GMTMinus6,
    /// Etc/GMT-7
    Etc__GMTMinus7,
    /// Etc/GMT-8
    Etc__GMTMinus8,
    /// Etc/GMT-9
    Etc__GMTMinus9,
    /// Etc/GMT0
    Etc__GMT0,
    /// Etc/Greenwich
    Etc__Greenwich,
    /// GMT
    GMT,
    /// GMT+0
    GMTPlus0,
    /// GMT-0
    GMTMinus0,
    /// GMT0
    GMT0,
    /// Greenwich
    Greenwich,
}
impl FromStr for Tz {
    #[cfg(feature = "std")]
    type Err = String;
    #[cfg(not(feature = "std"))]
    type Err = &'static str;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s {
            "Etc/GMT" => Ok(Tz::Etc__GMT),
            "Etc/GMT+0" => Ok(Tz::Etc__GMTPlus0),
            "Etc/GMT+1" => Ok(Tz::Etc__GMTPlus1),
            "Etc/GMT+10" => Ok(Tz::Etc__GMTPlus10),
            "Etc/GMT+11" => Ok(Tz::Etc__GMTPlus11),
            "Etc/GMT+12" => Ok(Tz::Etc__GMTPlus12),
            "Etc/GMT+2" => Ok(Tz::Etc__GMTPlus2),
            "Etc/GMT+3" => Ok(Tz::Etc__GMTPlus3),
            "Etc/GMT+4" => Ok(Tz::Etc__GMTPlus4),
            "Etc/GMT+5" => Ok(Tz::Etc__GMTPlus5),
            "Etc/GMT+6" => Ok(Tz::Etc__GMTPlus6),
            "Etc/GMT+7" => Ok(Tz::Etc__GMTPlus7),
            "Etc/GMT+8" => Ok(Tz::Etc__GMTPlus8),
            "Etc/GMT+9" => Ok(Tz::Etc__GMTPlus9),
            "Etc/GMT-0" => Ok(Tz::Etc__GMTMinus0),
            "Etc/GMT-1" => Ok(Tz::Etc__GMTMinus1),
            "Etc/GMT-10" => Ok(Tz::Etc__GMTMinus10),
            "Etc/GMT-11" => Ok(Tz::Etc__GMTMinus11),
            "Etc/GMT-12" => Ok(Tz::Etc__GMTMinus12),
            "Etc/GMT-13" => Ok(Tz::Etc__GMTMinus13),
            "Etc/GMT-14" => Ok(Tz::Etc__GMTMinus14),
            "Etc/GMT-2" => Ok(Tz::Etc__GMTMinus2),
            "Etc/GMT-3" => Ok(Tz::Etc__GMTMinus3),
            "Etc/GMT-4" => Ok(Tz::Etc__GMTMinus4),
            "Etc/GMT-5" => Ok(Tz::Etc__GMTMinus5),
            "Etc/GMT-6" => Ok(Tz::Etc__GMTMinus6),
            "Etc/GMT-7" => Ok(Tz::Etc__GMTMinus7),
            "Etc/GMT-8" => Ok(Tz::Etc__GMTMinus8),
            "Etc/GMT-9" => Ok(Tz::Etc__GMTMinus9),
            "Etc/GMT0" => Ok(Tz::Etc__GMT0),
            "Etc/Greenwich" => Ok(Tz::Etc__Greenwich),
            "GMT" => Ok(Tz::GMT),
            "GMT+0" => Ok(Tz::GMTPlus0),
            "GMT-0" => Ok(Tz::GMTMinus0),
            "GMT0" => Ok(Tz::GMT0),
            "Greenwich" => Ok(Tz::Greenwich),
            #[cfg(feature = "std")]
            s => Err(format!("'{}' is not a valid timezone", s.to_string())),
            #[cfg(not(feature = "std"))]
            _ => Err("received invalid timezone"),
        }
    }
}

impl Tz {
    pub fn name(self) -> &'static str {
        match self {
            Tz::Etc__GMT => "Etc/GMT",
            Tz::Etc__GMTPlus0 => "Etc/GMT+0",
            Tz::Etc__GMTPlus1 => "Etc/GMT+1",
            Tz::Etc__GMTPlus10 => "Etc/GMT+10",
            Tz::Etc__GMTPlus11 => "Etc/GMT+11",
            Tz::Etc__GMTPlus12 => "Etc/GMT+12",
            Tz::Etc__GMTPlus2 => "Etc/GMT+2",
            Tz::Etc__GMTPlus3 => "Etc/GMT+3",
            Tz::Etc__GMTPlus4 => "Etc/GMT+4",
            Tz::Etc__GMTPlus5 => "Etc/GMT+5",
            Tz::Etc__GMTPlus6 => "Etc/GMT+6",
            Tz::Etc__GMTPlus7 => "Etc/GMT+7",
            Tz::Etc__GMTPlus8 => "Etc/GMT+8",
            Tz::Etc__GMTPlus9 => "Etc/GMT+9",
            Tz::Etc__GMTMinus0 => "Etc/GMT-0",
            Tz::Etc__GMTMinus1 => "Etc/GMT-1",
            Tz::Etc__GMTMinus10 => "Etc/GMT-10",
            Tz::Etc__GMTMinus11 => "Etc/GMT-11",
            Tz::Etc__GMTMinus12 => "Etc/GMT-12",
            Tz::Etc__GMTMinus13 => "Etc/GMT-13",
            Tz::Etc__GMTMinus14 => "Etc/GMT-14",
            Tz::Etc__GMTMinus2 => "Etc/GMT-2",
            Tz::Etc__GMTMinus3 => "Etc/GMT-3",
            Tz::Etc__GMTMinus4 => "Etc/GMT-4",
            Tz::Etc__GMTMinus5 => "Etc/GMT-5",
            Tz::Etc__GMTMinus6 => "Etc/GMT-6",
            Tz::Etc__GMTMinus7 => "Etc/GMT-7",
            Tz::Etc__GMTMinus8 => "Etc/GMT-8",
            Tz::Etc__GMTMinus9 => "Etc/GMT-9",
            Tz::Etc__GMT0 => "Etc/GMT0",
            Tz::Etc__Greenwich => "Etc/Greenwich",
            Tz::GMT => "GMT",
            Tz::GMTPlus0 => "GMT+0",
            Tz::GMTMinus0 => "GMT-0",
            Tz::GMT0 => "GMT0",
            Tz::Greenwich => "Greenwich",
        }
    }
}

impl Debug for Tz {
    fn fmt(&self, f: &mut Formatter) -> Result<(), Error> {
        write!(f, "{}", self.name())
    }
}

impl TimeSpans for Tz {
    fn timespans(&self) -> FixedTimespanSet {
        match *self {
            Tz::Etc__GMT => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus0 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus1 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -3600,
                        dst_offset: 0,
                        name: "-01",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus10 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -36000,
                        dst_offset: 0,
                        name: "-10",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus11 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -39600,
                        dst_offset: 0,
                        name: "-11",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus12 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -43200,
                        dst_offset: 0,
                        name: "-12",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus2 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -7200,
                        dst_offset: 0,
                        name: "-02",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus3 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -10800,
                        dst_offset: 0,
                        name: "-03",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus4 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -14400,
                        dst_offset: 0,
                        name: "-04",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus5 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -18000,
                        dst_offset: 0,
                        name: "-05",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus6 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -21600,
                        dst_offset: 0,
                        name: "-06",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus7 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -25200,
                        dst_offset: 0,
                        name: "-07",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus8 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -28800,
                        dst_offset: 0,
                        name: "-08",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTPlus9 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -32400,
                        dst_offset: 0,
                        name: "-09",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus0 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus1 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 3600,
                        dst_offset: 0,
                        name: "+01",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus10 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 36000,
                        dst_offset: 0,
                        name: "+10",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus11 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 39600,
                        dst_offset: 0,
                        name: "+11",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus12 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 43200,
                        dst_offset: 0,
                        name: "+12",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus13 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 46800,
                        dst_offset: 0,
                        name: "+13",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus14 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 50400,
                        dst_offset: 0,
                        name: "+14",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus2 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 7200,
                        dst_offset: 0,
                        name: "+02",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus3 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 10800,
                        dst_offset: 0,
                        name: "+03",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus4 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 14400,
                        dst_offset: 0,
                        name: "+04",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus5 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 18000,
                        dst_offset: 0,
                        name: "+05",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus6 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 21600,
                        dst_offset: 0,
                        name: "+06",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus7 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 25200,
                        dst_offset: 0,
                        name: "+07",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus8 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 28800,
                        dst_offset: 0,
                        name: "+08",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMTMinus9 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 32400,
                        dst_offset: 0,
                        name: "+09",
                    },
                    rest: REST
                }
            },

            Tz::Etc__GMT0 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::Etc__Greenwich => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::GMT => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::GMTPlus0 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::GMTMinus0 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::GMT0 => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

            Tz::Greenwich => {
                const REST: &[(i64, FixedTimespan)] = &[
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "GMT",
                    },
                    rest: REST
                }
            },

         }
    }
}
/// An array of every known variant
///
/// Useful for iterating over known timezones:
///
/// ```
/// use chrono_tz::{TZ_VARIANTS, Tz};
/// assert!(TZ_VARIANTS.iter().any(|v| *v == Tz::UTC));
/// ```
pub static TZ_VARIANTS: [Tz; 36] = [
    Tz::Etc__GMT,
    Tz::Etc__GMTPlus0,
    Tz::Etc__GMTPlus1,
    Tz::Etc__GMTPlus10,
    Tz::Etc__GMTPlus11,
    Tz::Etc__GMTPlus12,
    Tz::Etc__GMTPlus2,
    Tz::Etc__GMTPlus3,
    Tz::Etc__GMTPlus4,
    Tz::Etc__GMTPlus5,
    Tz::Etc__GMTPlus6,
    Tz::Etc__GMTPlus7,
    Tz::Etc__GMTPlus8,
    Tz::Etc__GMTPlus9,
    Tz::Etc__GMTMinus0,
    Tz::Etc__GMTMinus1,
    Tz::Etc__GMTMinus10,
    Tz::Etc__GMTMinus11,
    Tz::Etc__GMTMinus12,
    Tz::Etc__GMTMinus13,
    Tz::Etc__GMTMinus14,
    Tz::Etc__GMTMinus2,
    Tz::Etc__GMTMinus3,
    Tz::Etc__GMTMinus4,
    Tz::Etc__GMTMinus5,
    Tz::Etc__GMTMinus6,
    Tz::Etc__GMTMinus7,
    Tz::Etc__GMTMinus8,
    Tz::Etc__GMTMinus9,
    Tz::Etc__GMT0,
    Tz::Etc__Greenwich,
    Tz::GMT,
    Tz::GMTPlus0,
    Tz::GMTMinus0,
    Tz::GMT0,
    Tz::Greenwich,
];