// REGEX = "Indian/.*"
use core::fmt::{Debug, Formatter, Error};
use core::str::FromStr;

use ::timezone_impl::{TimeSpans, FixedTimespanSet, FixedTimespan};

/// TimeZones built at compile time from the tz database
///
/// This implements [`chrono::TimeZone`] so that it may be used in and to
/// construct chrono's DateTime type. See the root module documentation
/// for details.
#[derive(Clone, Copy, PartialEq, Eq, Hash)]
pub enum Tz {
    /// Africa/Addis_Ababa
    Africa__Addis_Ababa,
    /// Africa/Asmara
    Africa__Asmara,
    /// Africa/Asmera
    Africa__Asmera,
    /// Africa/Dar_es_Salaam
    Africa__Dar_es_Salaam,
    /// Africa/Djibouti
    Africa__Djibouti,
    /// Africa/Kampala
    Africa__Kampala,
    /// Africa/Mogadishu
    Africa__Mogadishu,
    /// Africa/Nairobi
    Africa__Nairobi,
    /// Indian/Antananarivo
    Indian__Antananarivo,
    /// Indian/Chagos
    Indian__Chagos,
    /// Indian/Christmas
    Indian__Christmas,
    /// Indian/Cocos
    Indian__Cocos,
    /// Indian/Comoro
    Indian__Comoro,
    /// Indian/Kerguelen
    Indian__Kerguelen,
    /// Indian/Mahe
    Indian__Mahe,
    /// Indian/Maldives
    Indian__Maldives,
    /// Indian/Mauritius
    Indian__Mauritius,
    /// Indian/Mayotte
    Indian__Mayotte,
    /// Indian/Reunion
    Indian__Reunion,
}
impl FromStr for Tz {
    #[cfg(feature = "std")]
    type Err = String;
    #[cfg(not(feature = "std"))]
    type Err = &'static str;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s {
            "Africa/Addis_Ababa" => Ok(Tz::Africa__Addis_Ababa),
            "Africa/Asmara" => Ok(Tz::Africa__Asmara),
            "Africa/Asmera" => Ok(Tz::Africa__Asmera),
            "Africa/Dar_es_Salaam" => Ok(Tz::Africa__Dar_es_Salaam),
            "Africa/Djibouti" => Ok(Tz::Africa__Djibouti),
            "Africa/Kampala" => Ok(Tz::Africa__Kampala),
            "Africa/Mogadishu" => Ok(Tz::Africa__Mogadishu),
            "Africa/Nairobi" => Ok(Tz::Africa__Nairobi),
            "Indian/Antananarivo" => Ok(Tz::Indian__Antananarivo),
            "Indian/Chagos" => Ok(Tz::Indian__Chagos),
            "Indian/Christmas" => Ok(Tz::Indian__Christmas),
            "Indian/Cocos" => Ok(Tz::Indian__Cocos),
            "Indian/Comoro" => Ok(Tz::Indian__Comoro),
            "Indian/Kerguelen" => Ok(Tz::Indian__Kerguelen),
            "Indian/Mahe" => Ok(Tz::Indian__Mahe),
            "Indian/Maldives" => Ok(Tz::Indian__Maldives),
            "Indian/Mauritius" => Ok(Tz::Indian__Mauritius),
            "Indian/Mayotte" => Ok(Tz::Indian__Mayotte),
            "Indian/Reunion" => Ok(Tz::Indian__Reunion),
            #[cfg(feature = "std")]
            s => Err(format!("'{}' is not a valid timezone", s.to_string())),
            #[cfg(not(feature = "std"))]
            _ => Err("received invalid timezone"),
        }
    }
}

impl Tz {
    pub fn name(self) -> &'static str {
        match self {
            Tz::Africa__Addis_Ababa => "Africa/Addis_Ababa",
            Tz::Africa__Asmara => "Africa/Asmara",
            Tz::Africa__Asmera => "Africa/Asmera",
            Tz::Africa__Dar_es_Salaam => "Africa/Dar_es_Salaam",
            Tz::Africa__Djibouti => "Africa/Djibouti",
            Tz::Africa__Kampala => "Africa/Kampala",
            Tz::Africa__Mogadishu => "Africa/Mogadishu",
            Tz::Africa__Nairobi => "Africa/Nairobi",
            Tz::Indian__Antananarivo => "Indian/Antananarivo",
            Tz::Indian__Chagos => "Indian/Chagos",
            Tz::Indian__Christmas => "Indian/Christmas",
            Tz::Indian__Cocos => "Indian/Cocos",
            Tz::Indian__Comoro => "Indian/Comoro",
            Tz::Indian__Kerguelen => "Indian/Kerguelen",
            Tz::Indian__Mahe => "Indian/Mahe",
            Tz::Indian__Maldives => "Indian/Maldives",
            Tz::Indian__Mauritius => "Indian/Mauritius",
            Tz::Indian__Mayotte => "Indian/Mayotte",
            Tz::Indian__Reunion => "Indian/Reunion",
        }
    }
}

impl Debug for Tz {
    fn fmt(&self, f: &mut Formatter) -> Result<(), Error> {
        write!(f, "{}", self.name())
    }
}

impl TimeSpans for Tz {
    fn timespans(&self) -> FixedTimespanSet {
        match *self {
            Tz::Africa__Addis_Ababa => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Asmara => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Asmera => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Dar_es_Salaam => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Djibouti => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Kampala => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Mogadishu => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Africa__Nairobi => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Antananarivo => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Chagos => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1988167780, FixedTimespan { utc_offset: 18000, dst_offset: 0, name: "+05" }),
                    (820436400, FixedTimespan { utc_offset: 21600, dst_offset: 0, name: "+06" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 17380,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Christmas => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2364102172, FixedTimespan { utc_offset: 25200, dst_offset: 0, name: "+07" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 25372,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Cocos => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2209012060, FixedTimespan { utc_offset: 23400, dst_offset: 0, name: "+0630" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 23260,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Comoro => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Kerguelen => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-631152000, FixedTimespan { utc_offset: 18000, dst_offset: 0, name: "+05" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 0,
                        dst_offset: 0,
                        name: "-00",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Mahe => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2006653308, FixedTimespan { utc_offset: 14400, dst_offset: 0, name: "+04" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 13308,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Maldives => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2840158440, FixedTimespan { utc_offset: 17640, dst_offset: 0, name: "MMT" }),
                    (-315636840, FixedTimespan { utc_offset: 18000, dst_offset: 0, name: "+05" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 17640,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Mauritius => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1988164200, FixedTimespan { utc_offset: 14400, dst_offset: 0, name: "+04" }),
                    (403041600, FixedTimespan { utc_offset: 14400, dst_offset: 3600, name: "+05" }),
                    (417034800, FixedTimespan { utc_offset: 14400, dst_offset: 0, name: "+04" }),
                    (1224972000, FixedTimespan { utc_offset: 14400, dst_offset: 3600, name: "+05" }),
                    (1238274000, FixedTimespan { utc_offset: 14400, dst_offset: 0, name: "+04" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 13800,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Mayotte => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1309746436, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                    (-1262314800, FixedTimespan { utc_offset: 9000, dst_offset: 0, name: "+0230" }),
                    (-946780200, FixedTimespan { utc_offset: 9900, dst_offset: 0, name: "+0245" }),
                    (-315629100, FixedTimespan { utc_offset: 10800, dst_offset: 0, name: "EAT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 8836,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Indian__Reunion => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-1848886912, FixedTimespan { utc_offset: 14400, dst_offset: 0, name: "+04" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 13312,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

         }
    }
}
/// An array of every known variant
///
/// Useful for iterating over known timezones:
///
/// ```
/// use chrono_tz::{TZ_VARIANTS, Tz};
/// assert!(TZ_VARIANTS.iter().any(|v| *v == Tz::UTC));
/// ```
pub static TZ_VARIANTS: [Tz; 19] = [
    Tz::Africa__Addis_Ababa,
    Tz::Africa__Asmara,
    Tz::Africa__Asmera,
    Tz::Africa__Dar_es_Salaam,
    Tz::Africa__Djibouti,
    Tz::Africa__Kampala,
    Tz::Africa__Mogadishu,
    Tz::Africa__Nairobi,
    Tz::Indian__Antananarivo,
    Tz::Indian__Chagos,
    Tz::Indian__Christmas,
    Tz::Indian__Cocos,
    Tz::Indian__Comoro,
    Tz::Indian__Kerguelen,
    Tz::Indian__Mahe,
    Tz::Indian__Maldives,
    Tz::Indian__Mauritius,
    Tz::Indian__Mayotte,
    Tz::Indian__Reunion,
];