// REGEX = "Europe/London"
use core::fmt::{Debug, Formatter, Error};
use core::str::FromStr;

use ::timezone_impl::{TimeSpans, FixedTimespanSet, FixedTimespan};

/// TimeZones built at compile time from the tz database
///
/// This implements [`chrono::TimeZone`] so that it may be used in and to
/// construct chrono's DateTime type. See the root module documentation
/// for details.
#[derive(Clone, Copy, PartialEq, Eq, Hash)]
pub enum Tz {
    /// Europe/Belfast
    Europe__Belfast,
    /// Europe/Guernsey
    Europe__Guernsey,
    /// Europe/Isle_of_Man
    Europe__Isle_of_Man,
    /// Europe/Jersey
    Europe__Jersey,
    /// Europe/London
    Europe__London,
    /// GB
    GB,
    /// GB-Eire
    GBEire,
}
impl FromStr for Tz {
    #[cfg(feature = "std")]
    type Err = String;
    #[cfg(not(feature = "std"))]
    type Err = &'static str;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s {
            "Europe/Belfast" => Ok(Tz::Europe__Belfast),
            "Europe/Guernsey" => Ok(Tz::Europe__Guernsey),
            "Europe/Isle_of_Man" => Ok(Tz::Europe__Isle_of_Man),
            "Europe/Jersey" => Ok(Tz::Europe__Jersey),
            "Europe/London" => Ok(Tz::Europe__London),
            "GB" => Ok(Tz::GB),
            "GB-Eire" => Ok(Tz::GBEire),
            #[cfg(feature = "std")]
            s => Err(format!("'{}' is not a valid timezone", s.to_string())),
            #[cfg(not(feature = "std"))]
            _ => Err("received invalid timezone"),
        }
    }
}

impl Tz {
    pub fn name(self) -> &'static str {
        match self {
            Tz::Europe__Belfast => "Europe/Belfast",
            Tz::Europe__Guernsey => "Europe/Guernsey",
            Tz::Europe__Isle_of_Man => "Europe/Isle_of_Man",
            Tz::Europe__Jersey => "Europe/Jersey",
            Tz::Europe__London => "Europe/London",
            Tz::GB => "GB",
            Tz::GBEire => "GB-Eire",
        }
    }
}

impl Debug for Tz {
    fn fmt(&self, f: &mut Formatter) -> Result<(), Error> {
        write!(f, "{}", self.name())
    }
}

impl TimeSpans for Tz {
    fn timespans(&self) -> FixedTimespanSet {
        match *self {
            Tz::Europe__Belfast => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Europe__Guernsey => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Europe__Isle_of_Man => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Europe__Jersey => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Europe__London => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::GB => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::GBEire => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3852662325, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1691964000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1680472800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1664143200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1650146400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1633903200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1617487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1601848800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1586037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1570399200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1552168800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1538344800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1522533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1507500000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1490565600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1473631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1460930400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1442786400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1428876000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1410732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1396216800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1379282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1364767200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1348437600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1333317600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1315778400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1301263200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1284328800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1269813600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1253484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1238364000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1221429600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1206914400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1189980000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1175464800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1159135200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1143410400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1126476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1111960800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1095631200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1080511200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1063576800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1049061600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1032127200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-1017612000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-1001282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-986162400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-969228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-950479200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-942012000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-904518000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-896050800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-875487600, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-864601200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-844038000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-832546800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-812588400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-798073200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-781052400, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-772066800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-764805600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-748476000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-733356000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-719445600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-717030000, FixedTimespan { utc_offset: 0, dst_offset: 7200, name: "BDST" }),
                    (-706748400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-699487200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-687996000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-668037600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-654732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-636588000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-622072800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-605743200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-590623200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-574293600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-558568800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-542239200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-527119200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-512604000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-496274400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-481154400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-464220000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-449704800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-432165600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-417650400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-401320800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-386200800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-369266400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-354751200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-337816800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-323301600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-306972000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-291852000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-276732000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-257983200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-245282400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-226533600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-213228000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-195084000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-182383200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-163634400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-150933600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-132184800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-119484000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-100735200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-88034400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-68680800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (-59004000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (-37242000, FixedTimespan { utc_offset: 3600, dst_offset: 0, name: "BST" }),
                    (57718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (69818400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (89172000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (101268000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (120621600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (132717600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (152071200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (164167200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (183520800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (196221600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (214970400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (227671200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (246420000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (259120800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (278474400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (290570400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (309924000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (322020000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (341373600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (354675600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (372819600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (386125200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (404269200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (417574800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (435718800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (449024400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (467773200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (481078800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (499222800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (512528400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (530672400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (543978000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (562122000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (575427600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (593571600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (606877200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (625626000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (638326800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (657075600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (670381200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (688525200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (701830800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (719974800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (733280400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (751424400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (764730000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (782874000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (796179600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (814323600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (828234000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (846378000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (859683600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (877827600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (891133200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (909277200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (922582800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (941331600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (954032400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (972781200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (985482000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1004230800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1017536400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1035680400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1048986000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1067130000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1080435600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1099184400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1111885200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1130634000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1143334800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1162083600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1174784400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1193533200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1206838800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1224982800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1238288400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1256432400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1269738000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1288486800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1301187600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1319936400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1332637200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1351386000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1364691600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1382835600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1396141200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1414285200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1427590800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1445734800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1459040400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1477789200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1490490000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1509238800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1521939600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1540688400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1553994000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1572138000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1585443600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1603587600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1616893200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1635642000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1648342800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1667091600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1679792400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1698541200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1711846800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1729990800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1743296400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1761440400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1774746000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1792890000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1806195600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1824944400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1837645200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1856394000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1869094800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1887843600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1901149200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1919293200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1932598800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1950742800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1964048400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (1982797200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (1995498000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2014246800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2026947600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2045696400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2058397200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2077146000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2090451600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2108595600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2121901200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2140045200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2153350800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2172099600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2184800400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2203549200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2216250000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2234998800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2248304400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2266448400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2279754000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2297898000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2311203600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2329347600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2342653200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2361402000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2374102800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2392851600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2405552400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2424301200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2437606800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2455750800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2469056400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2487200400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2500506000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2519254800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2531955600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2550704400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2563405200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2582154000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2595459600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2613603600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2626909200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2645053200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2658358800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2676502800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2689808400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2708557200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2721258000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2740006800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2752707600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2771456400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2784762000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2802906000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2816211600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2834355600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2847661200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2866410000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2879110800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2897859600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2910560400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2929309200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2942010000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2960758800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (2974064400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (2992208400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3005514000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3023658000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3036963600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3055712400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3068413200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3087162000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3099862800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3118611600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3131917200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3150061200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3163366800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3181510800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3194816400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3212960400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3226266000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3245014800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3257715600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3276464400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3289165200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3307914000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3321219600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3339363600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3352669200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3370813200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3384118800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3402867600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3415568400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3434317200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3447018000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3465766800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3479072400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3497216400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3510522000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3528666000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3541971600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3560115600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3573421200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3592170000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3604870800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3623619600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3636320400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3655069200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3668374800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3686518800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3699824400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3717968400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3731274000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3750022800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3762723600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3781472400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3794173200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3812922000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3825622800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3844371600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3857677200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3875821200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3889126800, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3907270800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3920576400, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3939325200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3952026000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (3970774800, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (3983475600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4002224400, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4015530000, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4033674000, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4046979600, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4065123600, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                    (4078429200, FixedTimespan { utc_offset: 0, dst_offset: 3600, name: "BST" }),
                    (4096573200, FixedTimespan { utc_offset: 0, dst_offset: 0, name: "GMT" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -75,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

         }
    }
}
/// An array of every known variant
///
/// Useful for iterating over known timezones:
///
/// ```
/// use chrono_tz::{TZ_VARIANTS, Tz};
/// assert!(TZ_VARIANTS.iter().any(|v| *v == Tz::UTC));
/// ```
pub static TZ_VARIANTS: [Tz; 7] = [
    Tz::Europe__Belfast,
    Tz::Europe__Guernsey,
    Tz::Europe__Isle_of_Man,
    Tz::Europe__Jersey,
    Tz::Europe__London,
    Tz::GB,
    Tz::GBEire,
];