// REGEX = "US/.*"
use core::fmt::{Debug, Formatter, Error};
use core::str::FromStr;

use ::timezone_impl::{TimeSpans, FixedTimespanSet, FixedTimespan};

/// TimeZones built at compile time from the tz database
///
/// This implements [`chrono::TimeZone`] so that it may be used in and to
/// construct chrono's DateTime type. See the root module documentation
/// for details.
#[derive(Clone, Copy, PartialEq, Eq, Hash)]
pub enum Tz {
    /// America/Adak
    America__Adak,
    /// America/Anchorage
    America__Anchorage,
    /// America/Atka
    America__Atka,
    /// America/Chicago
    America__Chicago,
    /// America/Denver
    America__Denver,
    /// America/Detroit
    America__Detroit,
    /// America/Fort_Wayne
    America__Fort_Wayne,
    /// America/Indiana/Indianapolis
    America__Indiana__Indianapolis,
    /// America/Indiana/Knox
    America__Indiana__Knox,
    /// America/Indiana/Marengo
    America__Indiana__Marengo,
    /// America/Indiana/Petersburg
    America__Indiana__Petersburg,
    /// America/Indiana/Tell_City
    America__Indiana__Tell_City,
    /// America/Indiana/Vevay
    America__Indiana__Vevay,
    /// America/Indiana/Vincennes
    America__Indiana__Vincennes,
    /// America/Indiana/Winamac
    America__Indiana__Winamac,
    /// America/Indianapolis
    America__Indianapolis,
    /// America/Knox_IN
    America__Knox_IN,
    /// America/Los_Angeles
    America__Los_Angeles,
    /// America/New_York
    America__New_York,
    /// America/Phoenix
    America__Phoenix,
    /// America/Shiprock
    America__Shiprock,
    /// Navajo
    Navajo,
    /// Pacific/Honolulu
    Pacific__Honolulu,
    /// Pacific/Johnston
    Pacific__Johnston,
    /// Pacific/Midway
    Pacific__Midway,
    /// Pacific/Pago_Pago
    Pacific__Pago_Pago,
    /// Pacific/Samoa
    Pacific__Samoa,
    /// US/Alaska
    US__Alaska,
    /// US/Aleutian
    US__Aleutian,
    /// US/Arizona
    US__Arizona,
    /// US/Central
    US__Central,
    /// US/East-Indiana
    US__EastIndiana,
    /// US/Eastern
    US__Eastern,
    /// US/Hawaii
    US__Hawaii,
    /// US/Indiana-Starke
    US__IndianaStarke,
    /// US/Michigan
    US__Michigan,
    /// US/Mountain
    US__Mountain,
    /// US/Pacific
    US__Pacific,
    /// US/Pacific-New
    US__PacificNew,
    /// US/Samoa
    US__Samoa,
}
impl FromStr for Tz {
    #[cfg(feature = "std")]
    type Err = String;
    #[cfg(not(feature = "std"))]
    type Err = &'static str;
    fn from_str(s: &str) -> Result<Self, Self::Err> {
        match s {
            "America/Adak" => Ok(Tz::America__Adak),
            "America/Anchorage" => Ok(Tz::America__Anchorage),
            "America/Atka" => Ok(Tz::America__Atka),
            "America/Chicago" => Ok(Tz::America__Chicago),
            "America/Denver" => Ok(Tz::America__Denver),
            "America/Detroit" => Ok(Tz::America__Detroit),
            "America/Fort_Wayne" => Ok(Tz::America__Fort_Wayne),
            "America/Indiana/Indianapolis" => Ok(Tz::America__Indiana__Indianapolis),
            "America/Indiana/Knox" => Ok(Tz::America__Indiana__Knox),
            "America/Indiana/Marengo" => Ok(Tz::America__Indiana__Marengo),
            "America/Indiana/Petersburg" => Ok(Tz::America__Indiana__Petersburg),
            "America/Indiana/Tell_City" => Ok(Tz::America__Indiana__Tell_City),
            "America/Indiana/Vevay" => Ok(Tz::America__Indiana__Vevay),
            "America/Indiana/Vincennes" => Ok(Tz::America__Indiana__Vincennes),
            "America/Indiana/Winamac" => Ok(Tz::America__Indiana__Winamac),
            "America/Indianapolis" => Ok(Tz::America__Indianapolis),
            "America/Knox_IN" => Ok(Tz::America__Knox_IN),
            "America/Los_Angeles" => Ok(Tz::America__Los_Angeles),
            "America/New_York" => Ok(Tz::America__New_York),
            "America/Phoenix" => Ok(Tz::America__Phoenix),
            "America/Shiprock" => Ok(Tz::America__Shiprock),
            "Navajo" => Ok(Tz::Navajo),
            "Pacific/Honolulu" => Ok(Tz::Pacific__Honolulu),
            "Pacific/Johnston" => Ok(Tz::Pacific__Johnston),
            "Pacific/Midway" => Ok(Tz::Pacific__Midway),
            "Pacific/Pago_Pago" => Ok(Tz::Pacific__Pago_Pago),
            "Pacific/Samoa" => Ok(Tz::Pacific__Samoa),
            "US/Alaska" => Ok(Tz::US__Alaska),
            "US/Aleutian" => Ok(Tz::US__Aleutian),
            "US/Arizona" => Ok(Tz::US__Arizona),
            "US/Central" => Ok(Tz::US__Central),
            "US/East-Indiana" => Ok(Tz::US__EastIndiana),
            "US/Eastern" => Ok(Tz::US__Eastern),
            "US/Hawaii" => Ok(Tz::US__Hawaii),
            "US/Indiana-Starke" => Ok(Tz::US__IndianaStarke),
            "US/Michigan" => Ok(Tz::US__Michigan),
            "US/Mountain" => Ok(Tz::US__Mountain),
            "US/Pacific" => Ok(Tz::US__Pacific),
            "US/Pacific-New" => Ok(Tz::US__PacificNew),
            "US/Samoa" => Ok(Tz::US__Samoa),
            #[cfg(feature = "std")]
            s => Err(format!("'{}' is not a valid timezone", s.to_string())),
            #[cfg(not(feature = "std"))]
            _ => Err("received invalid timezone"),
        }
    }
}

impl Tz {
    pub fn name(self) -> &'static str {
        match self {
            Tz::America__Adak => "America/Adak",
            Tz::America__Anchorage => "America/Anchorage",
            Tz::America__Atka => "America/Atka",
            Tz::America__Chicago => "America/Chicago",
            Tz::America__Denver => "America/Denver",
            Tz::America__Detroit => "America/Detroit",
            Tz::America__Fort_Wayne => "America/Fort_Wayne",
            Tz::America__Indiana__Indianapolis => "America/Indiana/Indianapolis",
            Tz::America__Indiana__Knox => "America/Indiana/Knox",
            Tz::America__Indiana__Marengo => "America/Indiana/Marengo",
            Tz::America__Indiana__Petersburg => "America/Indiana/Petersburg",
            Tz::America__Indiana__Tell_City => "America/Indiana/Tell_City",
            Tz::America__Indiana__Vevay => "America/Indiana/Vevay",
            Tz::America__Indiana__Vincennes => "America/Indiana/Vincennes",
            Tz::America__Indiana__Winamac => "America/Indiana/Winamac",
            Tz::America__Indianapolis => "America/Indianapolis",
            Tz::America__Knox_IN => "America/Knox_IN",
            Tz::America__Los_Angeles => "America/Los_Angeles",
            Tz::America__New_York => "America/New_York",
            Tz::America__Phoenix => "America/Phoenix",
            Tz::America__Shiprock => "America/Shiprock",
            Tz::Navajo => "Navajo",
            Tz::Pacific__Honolulu => "Pacific/Honolulu",
            Tz::Pacific__Johnston => "Pacific/Johnston",
            Tz::Pacific__Midway => "Pacific/Midway",
            Tz::Pacific__Pago_Pago => "Pacific/Pago_Pago",
            Tz::Pacific__Samoa => "Pacific/Samoa",
            Tz::US__Alaska => "US/Alaska",
            Tz::US__Aleutian => "US/Aleutian",
            Tz::US__Arizona => "US/Arizona",
            Tz::US__Central => "US/Central",
            Tz::US__EastIndiana => "US/East-Indiana",
            Tz::US__Eastern => "US/Eastern",
            Tz::US__Hawaii => "US/Hawaii",
            Tz::US__IndianaStarke => "US/Indiana-Starke",
            Tz::US__Michigan => "US/Michigan",
            Tz::US__Mountain => "US/Mountain",
            Tz::US__Pacific => "US/Pacific",
            Tz::US__PacificNew => "US/Pacific-New",
            Tz::US__Samoa => "US/Samoa",
        }
    }
}

impl Debug for Tz {
    fn fmt(&self, f: &mut Formatter) -> Result<(), Error> {
        write!(f, "{}", self.name())
    }
}

impl TimeSpans for Tz {
    fn timespans(&self) -> FixedTimespanSet {
        match *self {
            Tz::America__Adak => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3225223727, FixedTimespan { utc_offset: -42398, dst_offset: 0, name: "LMT" }),
                    (-2188944802, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "NST" }),
                    (-880196400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "NWT" }),
                    (-769395600, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "NPT" }),
                    (-765374400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "NST" }),
                    (-86878800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (-21466800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (-5745600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (9982800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (25704000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (41432400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (57758400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (73486800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (89208000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (104936400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (120657600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (126709200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (152107200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (162392400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (183556800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (199285200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (215611200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (230734800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (247060800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (262789200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (278510400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (294238800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (309960000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (325688400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (341409600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (357138000, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (372859200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (388587600, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (404913600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (420037200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (436363200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (439034400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (452088000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (467809200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (483537600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (499258800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (514987200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (530708400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (544622400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (562158000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (576072000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (594212400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (607521600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (625662000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (638971200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (657111600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (671025600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (688561200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (702475200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (720010800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (733924800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (752065200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (765374400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (783514800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (796824000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (814964400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (828878400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (846414000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (860328000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (877863600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (891777600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (909313200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (923227200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (941367600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (954676800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (972817200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (986126400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1004266800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1018180800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1035716400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1049630400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1067166000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1081080000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1099220400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1112529600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1130670000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1143979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1162119600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1173614400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1194174000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1205064000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1225623600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1236513600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1257073200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1268568000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1289127600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1300017600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1320577200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1331467200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1352026800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1362916800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1383476400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1394366400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1414926000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1425816000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1446375600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1457870400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1478430000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1489320000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1509879600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1520769600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1541329200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1552219200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1572778800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1583668800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1604228400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1615723200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1636282800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1647172800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1667732400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1678622400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1699182000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1710072000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1730631600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1741521600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1762081200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1772971200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1793530800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1805025600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1825585200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1836475200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1857034800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1867924800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1888484400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1899374400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1919934000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1930824000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1951383600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1962878400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1983438000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1994328000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2014887600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2025777600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2046337200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2057227200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2077786800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2088676800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2109236400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2120126400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2140686000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2152180800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2172740400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2183630400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2204190000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2215080000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2235639600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2246529600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2267089200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2277979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2298538800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2309428800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2329988400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2341483200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2362042800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2372932800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2393492400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2404382400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2424942000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2435832000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2456391600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2467281600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2487841200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2499336000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2519895600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2530785600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2551345200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2562235200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2582794800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2593684800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2614244400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2625134400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2645694000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2656584000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2677143600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2688638400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2709198000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2720088000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2740647600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2751537600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2772097200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2782987200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2803546800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2814436800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2834996400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2846491200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2867050800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2877940800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2898500400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2909390400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2929950000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2940840000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2961399600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2972289600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2992849200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3003739200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3024298800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3035793600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3056353200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3067243200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3087802800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3098692800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3119252400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3130142400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3150702000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3161592000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3182151600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3193041600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3213601200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3225096000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3245655600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3256545600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3277105200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3287995200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3308554800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3319444800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3340004400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3350894400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3371454000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3382948800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3403508400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3414398400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3434958000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3445848000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3466407600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3477297600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3497857200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3508747200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3529306800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3540196800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3560756400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3572251200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3592810800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3603700800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3624260400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3635150400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3655710000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3666600000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3687159600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3698049600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3718609200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3730104000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3750663600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3761553600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3782113200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3793003200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3813562800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3824452800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3845012400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3855902400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3876462000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3887352000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3907911600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3919406400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3939966000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3950856000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3971415600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3982305600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4002865200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4013755200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4034314800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4045204800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4065764400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4076654400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4097214000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 44002,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Anchorage => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3225223727, FixedTimespan { utc_offset: -35976, dst_offset: 0, name: "LMT" }),
                    (-2188951224, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AST" }),
                    (-880200000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AWT" }),
                    (-769395600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "APT" }),
                    (-765378000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AST" }),
                    (-86882400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (-21470400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (-5749200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (9979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (25700400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (41428800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (57754800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (73483200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (89204400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (104932800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (120654000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (126705600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (152103600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (162388800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (183553200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (199281600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (215607600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (230731200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (247057200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (262785600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (278506800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (294235200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (309956400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (325684800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (341406000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (357134400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (372855600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (388584000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (404910000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (420033600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (436359600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "YST" }),
                    (439030800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (452084400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (467805600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (483534000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (499255200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (514983600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (530704800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (544618800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (562154400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (576068400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (594208800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (607518000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (625658400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (638967600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (657108000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (671022000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (688557600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (702471600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (720007200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (733921200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (752061600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (765370800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (783511200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (796820400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (814960800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (828874800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (846410400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (860324400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (877860000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (891774000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (909309600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (923223600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (941364000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (954673200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (972813600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (986122800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1004263200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1018177200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1035712800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1049626800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1067162400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1081076400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1099216800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1112526000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1130666400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1143975600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1162116000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1173610800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1194170400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1205060400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1225620000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1236510000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1257069600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1268564400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1289124000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1300014000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1320573600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1331463600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1352023200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1362913200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1383472800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1394362800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1414922400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1425812400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1446372000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1457866800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1478426400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1489316400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1509876000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1520766000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1541325600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1552215600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1572775200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1583665200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1604224800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1615719600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1636279200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1647169200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1667728800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1678618800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1699178400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1710068400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1730628000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1741518000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1762077600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1772967600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1793527200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1805022000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1825581600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1836471600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1857031200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1867921200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1888480800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1899370800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1919930400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1930820400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1951380000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1962874800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1983434400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1994324400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2014884000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2025774000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2046333600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2057223600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2077783200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2088673200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2109232800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2120122800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2140682400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2152177200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2172736800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2183626800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2204186400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2215076400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2235636000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2246526000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2267085600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2277975600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2298535200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2309425200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2329984800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2341479600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2362039200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2372929200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2393488800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2404378800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2424938400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2435828400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2456388000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2467278000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2487837600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2499332400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2519892000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2530782000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2551341600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2562231600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2582791200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2593681200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2614240800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2625130800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2645690400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2656580400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2677140000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2688634800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2709194400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2720084400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2740644000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2751534000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2772093600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2782983600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2803543200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2814433200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2834992800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2846487600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2867047200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2877937200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2898496800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2909386800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2929946400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2940836400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2961396000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2972286000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2992845600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3003735600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3024295200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3035790000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3056349600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3067239600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3087799200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3098689200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3119248800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3130138800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3150698400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3161588400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3182148000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3193038000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3213597600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3225092400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3245652000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3256542000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3277101600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3287991600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3308551200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3319441200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3340000800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3350890800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3371450400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3382945200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3403504800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3414394800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3434954400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3445844400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3466404000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3477294000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3497853600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3508743600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3529303200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3540193200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3560752800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3572247600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3592807200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3603697200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3624256800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3635146800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3655706400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3666596400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3687156000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3698046000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3718605600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3730100400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3750660000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3761550000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3782109600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3792999600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3813559200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3824449200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3845008800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3855898800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3876458400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3887348400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3907908000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3919402800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3939962400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3950852400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3971412000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3982302000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4002861600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (4013751600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4034311200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (4045201200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4065760800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (4076650800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4097210400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 50424,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Atka => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3225223727, FixedTimespan { utc_offset: -42398, dst_offset: 0, name: "LMT" }),
                    (-2188944802, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "NST" }),
                    (-880196400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "NWT" }),
                    (-769395600, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "NPT" }),
                    (-765374400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "NST" }),
                    (-86878800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (-21466800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (-5745600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (9982800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (25704000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (41432400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (57758400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (73486800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (89208000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (104936400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (120657600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (126709200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (152107200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (162392400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (183556800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (199285200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (215611200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (230734800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (247060800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (262789200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (278510400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (294238800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (309960000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (325688400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (341409600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (357138000, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (372859200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (388587600, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (404913600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (420037200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (436363200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (439034400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (452088000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (467809200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (483537600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (499258800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (514987200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (530708400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (544622400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (562158000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (576072000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (594212400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (607521600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (625662000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (638971200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (657111600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (671025600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (688561200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (702475200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (720010800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (733924800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (752065200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (765374400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (783514800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (796824000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (814964400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (828878400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (846414000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (860328000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (877863600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (891777600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (909313200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (923227200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (941367600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (954676800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (972817200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (986126400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1004266800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1018180800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1035716400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1049630400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1067166000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1081080000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1099220400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1112529600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1130670000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1143979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1162119600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1173614400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1194174000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1205064000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1225623600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1236513600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1257073200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1268568000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1289127600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1300017600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1320577200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1331467200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1352026800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1362916800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1383476400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1394366400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1414926000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1425816000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1446375600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1457870400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1478430000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1489320000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1509879600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1520769600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1541329200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1552219200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1572778800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1583668800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1604228400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1615723200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1636282800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1647172800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1667732400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1678622400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1699182000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1710072000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1730631600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1741521600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1762081200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1772971200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1793530800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1805025600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1825585200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1836475200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1857034800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1867924800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1888484400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1899374400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1919934000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1930824000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1951383600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1962878400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1983438000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1994328000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2014887600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2025777600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2046337200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2057227200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2077786800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2088676800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2109236400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2120126400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2140686000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2152180800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2172740400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2183630400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2204190000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2215080000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2235639600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2246529600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2267089200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2277979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2298538800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2309428800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2329988400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2341483200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2362042800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2372932800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2393492400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2404382400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2424942000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2435832000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2456391600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2467281600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2487841200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2499336000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2519895600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2530785600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2551345200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2562235200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2582794800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2593684800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2614244400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2625134400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2645694000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2656584000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2677143600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2688638400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2709198000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2720088000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2740647600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2751537600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2772097200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2782987200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2803546800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2814436800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2834996400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2846491200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2867050800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2877940800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2898500400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2909390400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2929950000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2940840000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2961399600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2972289600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2992849200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3003739200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3024298800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3035793600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3056353200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3067243200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3087802800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3098692800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3119252400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3130142400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3150702000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3161592000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3182151600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3193041600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3213601200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3225096000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3245655600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3256545600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3277105200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3287995200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3308554800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3319444800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3340004400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3350894400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3371454000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3382948800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3403508400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3414398400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3434958000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3445848000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3466407600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3477297600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3497857200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3508747200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3529306800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3540196800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3560756400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3572251200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3592810800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3603700800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3624260400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3635150400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3655710000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3666600000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3687159600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3698049600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3718609200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3730104000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3750663600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3761553600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3782113200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3793003200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3813562800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3824452800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3845012400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3855902400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3876462000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3887352000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3907911600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3919406400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3939966000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3950856000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3971415600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3982305600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4002865200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4013755200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4034314800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4045204800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4065764400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4076654400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4097214000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 44002,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Chicago => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1563724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1551632400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1538928000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1520182800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1504454400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1491757200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1473004800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1459702800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1441555200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1428253200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1410105600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1396803600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1378656000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1365354000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1347206400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1333904400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1315152000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1301850000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1283702400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1270400400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1252252800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1238950800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1220803200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1207501200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1189353600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1176051600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1157299200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1144602000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1125849600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1112547600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1094400000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1081098000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1067788800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1045414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1031500800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1018198800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1000051200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-986749200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-967996800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-955299600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-936547200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-923245200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-905097600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-891795600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-447267600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-415818000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-384368400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-352918800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-321469200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-289414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-226515600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-210787200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-179337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-163616400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-147888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-131562000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-116438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-100112400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-84384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-5763600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (9964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (25686000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (41414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (57740400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (73468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (89190000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (104918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (120639600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (126691200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (162374400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (183538800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (199267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (215593200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (230716800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (247042800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (262771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (278492400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (294220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (309942000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (325670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (341391600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (357120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (372841200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (388569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (404895600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (420019200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (436345200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (452073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (467794800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (483523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (499244400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (514972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (530694000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (544608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (562143600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (576057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (594198000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (607507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (625647600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (638956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (657097200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (671011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (688546800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (702460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (719996400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (733910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (752050800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (765360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (783500400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (796809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (814950000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (828864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (846399600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (860313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (877849200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (891763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (909298800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (923212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (941353200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (954662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (972802800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (986112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1004252400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1018166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1035702000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1049616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1067151600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1081065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1099206000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1112515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1130655600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1143964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1205049600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1225609200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1236499200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1257058800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1268553600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1289113200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1300003200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1320562800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1331452800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1352012400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1362902400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1383462000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1394352000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1414911600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1425801600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1446361200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1457856000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1478415600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1489305600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1509865200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1520755200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1541314800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1552204800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1572764400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1583654400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1604214000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1615708800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1636268400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1647158400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1667718000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1678608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1699167600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1710057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1730617200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1741507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1762066800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1772956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1793516400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1805011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1825570800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1836460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1857020400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1867910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1888470000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1899360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1919919600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1930809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1951369200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1962864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1983423600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1994313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2014873200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2025763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2046322800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2057212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2077772400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2088662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2109222000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2120112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2140671600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2152166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2172726000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2183616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2204175600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2215065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2235625200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2246515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2267074800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2277964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2298524400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2309414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2329974000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2341468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2362028400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2372918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2393478000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2404368000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2424927600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2435817600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2456377200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2467267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2487826800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2499321600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2519881200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2530771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2551330800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2562220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2582780400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2593670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2614230000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2625120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2645679600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2656569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2677129200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2688624000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2709183600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2720073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2740633200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2751523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2772082800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2782972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2803532400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2814422400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2834982000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2846476800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2867036400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2877926400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2898486000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2909376000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2929935600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2940825600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2961385200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2972275200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2992834800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3003724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3024284400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3035779200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3056338800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3067228800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3087788400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3098678400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3119238000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3130128000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3150687600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3161577600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3182137200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3193027200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3213586800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3225081600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3245641200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3256531200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3277090800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3287980800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3308540400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3319430400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3339990000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3350880000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3371439600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3382934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3403494000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3414384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3434943600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3445833600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3466393200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3477283200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3497842800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3508732800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3529292400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3540182400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3560742000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3572236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3592796400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3603686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3624246000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3635136000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3655695600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3666585600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3687145200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3698035200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3718594800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3730089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3750649200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3761539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3782098800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3792988800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3813548400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3824438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3844998000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3855888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3876447600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3887337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3907897200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3919392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3939951600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3950841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3971401200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3982291200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4002850800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4013740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4034300400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4045190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4065750000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4076640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4097199600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -21036,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Denver => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717643600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1633273200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1615132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1601823600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1583683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1570374000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1551628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1538924400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1534089600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-880210800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-769395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MPT" }),
                    (-765388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-147884400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-131558400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-116434800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-100108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-84380400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-68659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-52930800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-37209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-21481200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-5760000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (9968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (25689600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (41418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (57744000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (73472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (89193600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (104922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (120643200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (126694800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (152092800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (162378000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (183542400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (199270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (215596800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (230720400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (247046400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (262774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (278496000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (294224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (309945600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (325674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (341395200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (357123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (372844800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (388573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (404899200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (420022800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (436348800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (452077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (467798400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (483526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (499248000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (514976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (530697600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (544611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (562147200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (576061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (594201600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (607510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (625651200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (638960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (657100800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (671014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (688550400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (702464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (720000000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (733914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (752054400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (765363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (783504000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (796813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (814953600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (828867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (846403200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (860317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (877852800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (891766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (909302400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (923216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (941356800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (954666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (972806400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (986115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1004256000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1018170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1035705600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1049619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1067155200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1081069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1099209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1112518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1130659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1143968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1162108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1173603600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1194163200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1205053200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1225612800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1236502800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1257062400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1268557200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1289116800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1300006800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1320566400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1331456400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1352016000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1362906000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1383465600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1394355600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1414915200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1425805200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1446364800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1457859600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1478419200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1489309200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1509868800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1520758800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1541318400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1552208400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1572768000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1583658000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1604217600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1615712400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1636272000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1647162000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1667721600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1678611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1699171200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1710061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1730620800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1741510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1762070400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1772960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1793520000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1805014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1825574400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1836464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1857024000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1867914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1888473600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1899363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1919923200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1930813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1951372800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1962867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1983427200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1994317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2014876800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2025766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2046326400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2057216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2077776000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2088666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2109225600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2120115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2140675200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2152170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2172729600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2183619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2204179200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2215069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2235628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2246518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2267078400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2277968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2298528000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2309418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2329977600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2341472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2362032000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2372922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2393481600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2404371600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2424931200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2435821200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2456380800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2467270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2487830400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2499325200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2519884800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2530774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2551334400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2562224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2582784000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2593674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2614233600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2625123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2645683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2656573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2677132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2688627600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2709187200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2720077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2740636800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2751526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2772086400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2782976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2803536000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2814426000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2834985600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2846480400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2867040000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2877930000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2898489600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2909379600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2929939200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2940829200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2961388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2972278800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2992838400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3003728400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3024288000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3035782800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3056342400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3067232400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3087792000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3098682000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3119241600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3130131600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3150691200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3161581200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3182140800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3193030800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3213590400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3225085200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3245644800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3256534800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3277094400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3287984400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3308544000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3319434000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3339993600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3350883600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3371443200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3382938000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3403497600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3414387600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3434947200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3445837200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3466396800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3477286800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3497846400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3508736400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3529296000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3540186000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3560745600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3572240400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3592800000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3603690000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3624249600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3635139600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3655699200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3666589200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3687148800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3698038800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3718598400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3730093200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3750652800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3761542800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3782102400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3792992400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3813552000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3824442000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3845001600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3855891600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3876451200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3887341200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3907900800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3919395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3939955200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3950845200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3971404800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3982294800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4002854400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4013744400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4034304000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4045194000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4065753600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4076643600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4097203200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -25196,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Detroit => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2051202469, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1724083200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-880218000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EWT" }),
                    (-769395600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EPT" }),
                    (-765396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-684349200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-671047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-80506740, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-68666400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-52938000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-37216800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (104914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (120636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (126687600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (152085600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (167814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (183535200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (199263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (215589600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (230713200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (247039200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (262767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (278488800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (294217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (309938400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (325666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (341388000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (357116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (372837600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (388566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (404892000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (420015600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (436341600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (452070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (467791200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (483519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (499240800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (514969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (530690400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (544604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (562140000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (576054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (594194400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (607503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (625644000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (638953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (657093600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (671007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (688543200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (702457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (719992800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (733906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (752047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (765356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (783496800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (796806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (814946400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (828860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (846396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (860310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (877845600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (891759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (909295200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (923209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (941349600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (954658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (972799200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (986108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1004248800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1018162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1035698400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1049612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1067148000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1081062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1099202400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1112511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1130652000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -19931,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Fort_Wayne => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-900259200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-891795600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20678,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Indianapolis => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-900259200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-891795600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20678,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Knox => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-447267600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-415818000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-321469200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-289414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-84384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-5763600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (9964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (25686000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (41414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (57740400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (73468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (89190000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (104918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (120639600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (126691200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (162374400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (183538800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (199267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (215593200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (230716800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (247042800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (262771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (278492400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (294220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (309942000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (325670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (341391600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (357120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (372841200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (388569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (404895600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (420019200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (436345200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (452073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (467794800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (483523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (499244400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (514972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (530694000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (544608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (562143600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (576057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (594198000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (607507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (625647600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (638956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (657097200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (671011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (688546800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1205049600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1225609200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1236499200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1257058800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1268553600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1289113200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1300003200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1320562800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1331452800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1352012400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1362902400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1383462000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1394352000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1414911600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1425801600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1446361200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1457856000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1478415600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1489305600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1509865200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1520755200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1541314800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1552204800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1572764400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1583654400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1604214000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1615708800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1636268400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1647158400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1667718000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1678608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1699167600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1710057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1730617200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1741507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1762066800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1772956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1793516400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1805011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1825570800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1836460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1857020400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1867910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1888470000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1899360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1919919600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1930809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1951369200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1962864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1983423600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1994313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2014873200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2025763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2046322800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2057212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2077772400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2088662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2109222000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2120112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2140671600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2152166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2172726000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2183616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2204175600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2215065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2235625200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2246515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2267074800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2277964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2298524400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2309414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2329974000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2341468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2362028400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2372918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2393478000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2404368000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2424927600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2435817600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2456377200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2467267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2487826800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2499321600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2519881200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2530771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2551330800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2562220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2582780400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2593670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2614230000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2625120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2645679600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2656569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2677129200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2688624000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2709183600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2720073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2740633200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2751523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2772082800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2782972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2803532400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2814422400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2834982000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2846476800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2867036400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2877926400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2898486000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2909376000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2929935600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2940825600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2961385200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2972275200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2992834800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3003724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3024284400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3035779200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3056338800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3067228800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3087788400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3098678400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3119238000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3130128000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3150687600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3161577600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3182137200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3193027200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3213586800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3225081600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3245641200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3256531200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3277090800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3287980800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3308540400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3319430400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3339990000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3350880000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3371439600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3382934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3403494000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3414384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3434943600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3445833600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3466393200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3477283200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3497842800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3508732800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3529292400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3540182400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3560742000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3572236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3592796400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3603686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3624246000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3635136000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3655695600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3666585600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3687145200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3698035200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3718594800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3730089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3750649200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3761539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3782098800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3792988800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3813548400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3824438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3844998000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3855888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3876447600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3887337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3907897200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3919392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3939951600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3950841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3971401200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3982291200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4002850800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4013740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4034300400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4045190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4065750000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4076640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4097199600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20790,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Marengo => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-450291600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-418237200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-323888400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-292438800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (41410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (57736800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (73465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (89186400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (104914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (120636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (126687600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (162370800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (183535200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20723,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Petersburg => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-462996000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-450291600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-418237200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-323888400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-292438800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-226515600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-210787200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-179337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-163616400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-147888000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-100112400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-84384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-5763600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (9964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (25686000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (41414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (57740400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (73468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (89190000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (104918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (120639600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (126691200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (162374400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (183538800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (199267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (215593200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (230716800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (247042800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20947,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Tell_City => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-462996000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-450291600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-418237200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-323888400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-292438800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-226515600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-210787200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-179337600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1205049600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1225609200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1236499200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1257058800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1268553600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1289113200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1300003200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1320562800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1331452800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1352012400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1362902400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1383462000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1394352000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1414911600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1425801600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1446361200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1457856000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1478415600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1489305600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1509865200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1520755200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1541314800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1552204800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1572764400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1583654400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1604214000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1615708800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1636268400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1647158400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1667718000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1678608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1699167600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1710057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1730617200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1741507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1762066800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1772956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1793516400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1805011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1825570800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1836460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1857020400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1867910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1888470000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1899360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1919919600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1930809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1951369200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1962864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1983423600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1994313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2014873200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2025763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2046322800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2057212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2077772400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2088662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2109222000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2120112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2140671600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2152166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2172726000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2183616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2204175600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2215065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2235625200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2246515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2267074800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2277964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2298524400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2309414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2329974000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2341468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2362028400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2372918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2393478000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2404368000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2424927600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2435817600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2456377200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2467267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2487826800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2499321600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2519881200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2530771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2551330800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2562220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2582780400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2593670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2614230000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2625120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2645679600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2656569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2677129200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2688624000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2709183600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2720073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2740633200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2751523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2772082800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2782972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2803532400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2814422400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2834982000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2846476800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2867036400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2877926400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2898486000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2909376000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2929935600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2940825600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2961385200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2972275200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2992834800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3003724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3024284400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3035779200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3056338800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3067228800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3087788400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3098678400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3119238000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3130128000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3150687600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3161577600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3182137200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3193027200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3213586800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3225081600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3245641200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3256531200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3277090800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3287980800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3308540400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3319430400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3339990000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3350880000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3371439600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3382934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3403494000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3414384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3434943600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3445833600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3466393200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3477283200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3497842800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3508732800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3529292400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3540182400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3560742000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3572236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3592796400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3603686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3624246000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3635136000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3655695600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3666585600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3687145200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3698035200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3718594800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3730089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3750649200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3761539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3782098800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3792988800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3813548400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3824438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3844998000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3855888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3876447600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3887337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3907897200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3919392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3939951600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3950841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3971401200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3982291200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4002850800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4013740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4034300400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4045190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4065750000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4076640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4097199600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20823,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Vevay => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (41410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (57736800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (73465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (89186400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20416,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Vincennes => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-462996000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-450291600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-418237200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-323888400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-289414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-260989200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-226515600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-210787200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-179337600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -21007,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indiana__Winamac => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-447267600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-415818000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-323888400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-292438800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20785,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Indianapolis => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-900259200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-891795600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20678,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Knox_IN => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-447267600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-415818000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-321469200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-289414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-84384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-5763600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (9964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (25686000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (41414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (57740400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (73468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (89190000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (104918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (120639600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (126691200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (162374400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (183538800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (199267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (215593200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (230716800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (247042800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (262771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (278492400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (294220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (309942000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (325670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (341391600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (357120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (372841200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (388569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (404895600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (420019200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (436345200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (452073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (467794800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (483523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (499244400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (514972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (530694000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (544608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (562143600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (576057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (594198000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (607507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (625647600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (638956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (657097200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (671011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (688546800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1205049600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1225609200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1236499200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1257058800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1268553600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1289113200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1300003200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1320562800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1331452800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1352012400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1362902400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1383462000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1394352000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1414911600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1425801600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1446361200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1457856000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1478415600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1489305600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1509865200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1520755200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1541314800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1552204800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1572764400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1583654400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1604214000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1615708800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1636268400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1647158400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1667718000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1678608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1699167600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1710057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1730617200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1741507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1762066800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1772956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1793516400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1805011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1825570800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1836460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1857020400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1867910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1888470000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1899360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1919919600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1930809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1951369200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1962864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1983423600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1994313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2014873200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2025763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2046322800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2057212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2077772400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2088662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2109222000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2120112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2140671600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2152166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2172726000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2183616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2204175600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2215065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2235625200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2246515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2267074800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2277964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2298524400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2309414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2329974000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2341468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2362028400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2372918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2393478000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2404368000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2424927600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2435817600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2456377200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2467267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2487826800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2499321600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2519881200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2530771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2551330800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2562220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2582780400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2593670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2614230000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2625120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2645679600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2656569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2677129200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2688624000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2709183600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2720073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2740633200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2751523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2772082800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2782972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2803532400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2814422400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2834982000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2846476800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2867036400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2877926400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2898486000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2909376000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2929935600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2940825600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2961385200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2972275200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2992834800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3003724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3024284400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3035779200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3056338800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3067228800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3087788400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3098678400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3119238000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3130128000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3150687600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3161577600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3182137200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3193027200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3213586800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3225081600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3245641200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3256531200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3277090800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3287980800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3308540400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3319430400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3339990000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3350880000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3371439600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3382934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3403494000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3414384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3434943600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3445833600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3466393200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3477283200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3497842800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3508732800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3529292400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3540182400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3560742000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3572236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3592796400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3603686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3624246000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3635136000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3655695600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3666585600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3687145200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3698035200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3718594800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3730089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3750649200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3761539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3782098800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3792988800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3813548400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3824438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3844998000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3855888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3876447600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3887337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3907897200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3919392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3939951600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3950841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3971401200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3982291200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4002850800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4013740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4034300400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4045190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4065750000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4076640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4097199600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20790,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Los_Angeles => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717640000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-1633269600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-1615129200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-1601820000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-1583679600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-880207200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PWT" }),
                    (-769395600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PPT" }),
                    (-765385200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-687967140, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-662655600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-620838000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-608137200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-589388400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-576082800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-557938800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-544633200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-526489200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-513183600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-495039600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-481734000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-463590000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-450284400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-431535600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-418230000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-400086000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-386780400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-368636400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-355330800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-337186800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-323881200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-305737200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-292431600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-273682800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-260982000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-242233200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-226508400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-210783600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-195058800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-179334000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-163609200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-147884400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-131554800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-116434800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-100105200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-84376800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-68655600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-52927200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-37206000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-21477600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-5756400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (9972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (25693200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (41421600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (57747600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (73476000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (89197200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (104925600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (120646800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (126698400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (152096400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (162381600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (183546000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (199274400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (215600400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (230724000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (247050000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (262778400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (278499600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (294228000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (309949200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (325677600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (341398800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (357127200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (372848400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (388576800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (404902800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (420026400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (436352400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (452080800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (467802000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (483530400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (499251600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (514980000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (530701200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (544615200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (562150800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (576064800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (594205200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (607514400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (625654800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (638964000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (657104400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (671018400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (688554000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (702468000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (720003600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (733917600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (752058000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (765367200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (783507600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (796816800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (814957200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (828871200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (846406800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (860320800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (877856400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (891770400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (909306000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (923220000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (941360400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (954669600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (972810000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (986119200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1004259600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1018173600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1035709200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1049623200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1067158800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1081072800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1099213200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1112522400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1130662800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1143972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1162112400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1173607200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1194166800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1205056800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1225616400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1236506400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1257066000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1268560800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1289120400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1300010400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1320570000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1331460000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1352019600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1362909600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1383469200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1394359200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1414918800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1425808800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1446368400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1457863200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1478422800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1489312800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1509872400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1520762400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1541322000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1552212000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1572771600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1583661600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1604221200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1615716000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1636275600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1647165600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1667725200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1678615200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1699174800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1710064800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1730624400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1741514400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1762074000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1772964000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1793523600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1805018400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1825578000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1836468000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1857027600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1867917600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1888477200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1899367200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1919926800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1930816800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1951376400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1962871200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1983430800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1994320800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2014880400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2025770400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2046330000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2057220000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2077779600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2088669600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2109229200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2120119200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2140678800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2152173600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2172733200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2183623200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2204182800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2215072800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2235632400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2246522400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2267082000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2277972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2298531600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2309421600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2329981200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2341476000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2362035600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2372925600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2393485200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2404375200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2424934800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2435824800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2456384400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2467274400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2487834000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2499328800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2519888400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2530778400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2551338000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2562228000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2582787600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2593677600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2614237200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2625127200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2645686800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2656576800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2677136400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2688631200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2709190800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2720080800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2740640400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2751530400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2772090000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2782980000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2803539600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2814429600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2834989200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2846484000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2867043600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2877933600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2898493200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2909383200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2929942800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2940832800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2961392400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2972282400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2992842000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3003732000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3024291600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3035786400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3056346000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3067236000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3087795600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3098685600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3119245200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3130135200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3150694800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3161584800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3182144400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3193034400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3213594000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3225088800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3245648400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3256538400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3277098000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3287988000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3308547600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3319437600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3339997200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3350887200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3371446800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3382941600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3403501200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3414391200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3434950800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3445840800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3466400400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3477290400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3497850000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3508740000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3529299600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3540189600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3560749200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3572244000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3592803600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3603693600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3624253200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3635143200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3655702800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3666592800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3687152400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3698042400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3718602000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3730096800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3750656400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3761546400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3782106000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3792996000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3813555600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3824445600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3845005200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3855895200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3876454800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3887344800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3907904400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3919399200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3939958800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3950848800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3971408400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3982298400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4002858000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4013748000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4034307600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4045197600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4065757200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4076647200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4097206800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -28378,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__New_York => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717650800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1633280400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1615140000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1601830800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1583690400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1570381200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1551636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1536512400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1523210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1504458000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1491760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1473008400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1459706400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1441558800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1428256800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1410109200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1396807200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1378659600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1365357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1347210000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1333908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1315155600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1301853600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1283706000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1270404000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1252256400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1238954400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1220806800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1207504800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1189357200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1176055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1157302800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1144605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1125853200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1112551200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1094403600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1081101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1062954000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1049652000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1031504400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1018202400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1000054800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-986752800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-968000400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-955303200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-936550800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-923248800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-905101200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-891799200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-880218000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EWT" }),
                    (-769395600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EPT" }),
                    (-765396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-747248400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-733946400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-715798800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-702496800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-684349200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-671047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-652899600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-639597600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-620845200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-608148000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-589395600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-576093600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-557946000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-544644000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-526496400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-513194400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-495046800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-481744800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-463597200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-447271200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-431542800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-415821600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-400093200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-384372000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-368643600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-352922400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-337194000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-321472800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-305744400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-289418400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-273690000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-257968800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-242240400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-226519200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-210790800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-195069600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-179341200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-163620000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-147891600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-131565600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-116442000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-100116000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-84387600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-68666400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-52938000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-37216800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (41410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (57736800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (73465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (89186400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (104914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (120636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (126687600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (152085600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (162370800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (183535200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (199263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (215589600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (230713200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (247039200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (262767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (278488800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (294217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (309938400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (325666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (341388000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (357116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (372837600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (388566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (404892000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (420015600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (436341600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (452070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (467791200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (483519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (499240800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (514969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (530690400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (544604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (562140000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (576054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (594194400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (607503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (625644000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (638953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (657093600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (671007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (688543200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (702457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (719992800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (733906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (752047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (765356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (783496800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (796806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (814946400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (828860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (846396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (860310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (877845600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (891759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (909295200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (923209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (941349600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (954658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (972799200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (986108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1004248800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1018162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1035698400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1049612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1067148000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1081062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1099202400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1112511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1130652000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -17762,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Phoenix => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717643600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1633273200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1615132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1601823600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1583683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-880210800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-820519140, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-812653140, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-796845540, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-84380400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-68659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -26898,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::America__Shiprock => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717643600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1633273200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1615132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1601823600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1583683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1570374000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1551628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1538924400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1534089600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-880210800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-769395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MPT" }),
                    (-765388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-147884400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-131558400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-116434800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-100108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-84380400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-68659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-52930800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-37209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-21481200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-5760000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (9968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (25689600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (41418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (57744000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (73472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (89193600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (104922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (120643200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (126694800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (152092800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (162378000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (183542400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (199270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (215596800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (230720400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (247046400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (262774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (278496000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (294224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (309945600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (325674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (341395200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (357123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (372844800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (388573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (404899200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (420022800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (436348800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (452077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (467798400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (483526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (499248000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (514976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (530697600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (544611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (562147200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (576061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (594201600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (607510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (625651200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (638960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (657100800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (671014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (688550400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (702464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (720000000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (733914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (752054400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (765363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (783504000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (796813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (814953600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (828867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (846403200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (860317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (877852800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (891766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (909302400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (923216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (941356800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (954666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (972806400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (986115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1004256000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1018170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1035705600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1049619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1067155200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1081069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1099209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1112518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1130659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1143968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1162108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1173603600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1194163200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1205053200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1225612800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1236502800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1257062400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1268557200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1289116800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1300006800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1320566400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1331456400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1352016000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1362906000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1383465600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1394355600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1414915200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1425805200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1446364800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1457859600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1478419200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1489309200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1509868800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1520758800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1541318400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1552208400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1572768000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1583658000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1604217600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1615712400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1636272000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1647162000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1667721600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1678611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1699171200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1710061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1730620800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1741510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1762070400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1772960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1793520000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1805014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1825574400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1836464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1857024000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1867914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1888473600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1899363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1919923200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1930813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1951372800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1962867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1983427200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1994317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2014876800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2025766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2046326400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2057216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2077776000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2088666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2109225600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2120115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2140675200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2152170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2172729600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2183619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2204179200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2215069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2235628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2246518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2267078400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2277968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2298528000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2309418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2329977600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2341472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2362032000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2372922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2393481600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2404371600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2424931200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2435821200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2456380800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2467270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2487830400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2499325200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2519884800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2530774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2551334400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2562224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2582784000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2593674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2614233600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2625123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2645683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2656573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2677132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2688627600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2709187200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2720077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2740636800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2751526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2772086400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2782976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2803536000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2814426000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2834985600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2846480400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2867040000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2877930000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2898489600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2909379600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2929939200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2940829200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2961388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2972278800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2992838400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3003728400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3024288000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3035782800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3056342400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3067232400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3087792000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3098682000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3119241600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3130131600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3150691200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3161581200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3182140800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3193030800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3213590400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3225085200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3245644800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3256534800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3277094400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3287984400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3308544000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3319434000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3339993600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3350883600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3371443200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3382938000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3403497600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3414387600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3434947200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3445837200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3466396800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3477286800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3497846400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3508736400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3529296000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3540186000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3560745600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3572240400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3592800000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3603690000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3624249600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3635139600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3655699200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3666589200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3687148800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3698038800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3718598400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3730093200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3750652800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3761542800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3782102400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3792992400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3813552000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3824442000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3845001600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3855891600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3876451200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3887341200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3907900800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3919395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3939955200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3950845200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3971404800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3982294800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4002854400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4013744400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4034304000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4045194000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4065753600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4076643600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4097203200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -25196,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Navajo => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717643600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1633273200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1615132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1601823600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1583683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1570374000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1551628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1538924400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1534089600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-880210800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-769395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MPT" }),
                    (-765388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-147884400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-131558400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-116434800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-100108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-84380400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-68659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-52930800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-37209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-21481200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-5760000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (9968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (25689600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (41418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (57744000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (73472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (89193600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (104922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (120643200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (126694800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (152092800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (162378000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (183542400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (199270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (215596800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (230720400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (247046400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (262774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (278496000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (294224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (309945600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (325674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (341395200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (357123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (372844800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (388573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (404899200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (420022800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (436348800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (452077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (467798400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (483526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (499248000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (514976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (530697600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (544611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (562147200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (576061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (594201600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (607510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (625651200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (638960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (657100800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (671014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (688550400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (702464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (720000000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (733914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (752054400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (765363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (783504000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (796813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (814953600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (828867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (846403200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (860317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (877852800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (891766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (909302400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (923216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (941356800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (954666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (972806400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (986115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1004256000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1018170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1035705600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1049619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1067155200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1081069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1099209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1112518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1130659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1143968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1162108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1173603600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1194163200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1205053200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1225612800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1236502800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1257062400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1268557200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1289116800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1300006800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1320566400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1331456400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1352016000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1362906000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1383465600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1394355600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1414915200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1425805200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1446364800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1457859600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1478419200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1489309200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1509868800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1520758800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1541318400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1552208400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1572768000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1583658000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1604217600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1615712400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1636272000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1647162000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1667721600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1678611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1699171200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1710061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1730620800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1741510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1762070400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1772960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1793520000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1805014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1825574400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1836464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1857024000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1867914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1888473600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1899363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1919923200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1930813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1951372800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1962867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1983427200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1994317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2014876800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2025766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2046326400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2057216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2077776000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2088666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2109225600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2120115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2140675200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2152170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2172729600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2183619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2204179200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2215069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2235628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2246518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2267078400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2277968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2298528000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2309418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2329977600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2341472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2362032000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2372922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2393481600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2404371600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2424931200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2435821200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2456380800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2467270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2487830400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2499325200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2519884800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2530774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2551334400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2562224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2582784000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2593674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2614233600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2625123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2645683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2656573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2677132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2688627600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2709187200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2720077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2740636800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2751526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2772086400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2782976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2803536000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2814426000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2834985600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2846480400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2867040000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2877930000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2898489600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2909379600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2929939200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2940829200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2961388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2972278800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2992838400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3003728400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3024288000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3035782800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3056342400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3067232400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3087792000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3098682000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3119241600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3130131600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3150691200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3161581200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3182140800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3193030800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3213590400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3225085200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3245644800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3256534800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3277094400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3287984400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3308544000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3319434000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3339993600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3350883600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3371443200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3382938000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3403497600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3414387600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3434947200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3445837200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3466396800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3477286800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3497846400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3508736400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3529296000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3540186000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3560745600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3572240400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3592800000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3603690000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3624249600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3635139600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3655699200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3666589200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3687148800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3698038800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3718598400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3730093200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3750652800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3761542800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3782102400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3792992400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3813552000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3824442000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3845001600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3855891600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3876451200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3887341200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3907900800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3919395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3939955200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3950845200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3971404800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3982294800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4002854400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4013744400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4034304000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4045194000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4065753600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4076643600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4097203200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -25196,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Pacific__Honolulu => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2334101314, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-1157283000, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HDT" }),
                    (-1155436200, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-880198200, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HWT" }),
                    (-769395600, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HPT" }),
                    (-765376200, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-712150200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -37886,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Pacific__Johnston => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2334101314, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-1157283000, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HDT" }),
                    (-1155436200, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-880198200, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HWT" }),
                    (-769395600, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HPT" }),
                    (-765376200, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-712150200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -37886,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Pacific__Midway => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2445424632, FixedTimespan { utc_offset: -40968, dst_offset: 0, name: "LMT" }),
                    (-1861879032, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "SST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 45432,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Pacific__Pago_Pago => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2445424632, FixedTimespan { utc_offset: -40968, dst_offset: 0, name: "LMT" }),
                    (-1861879032, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "SST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 45432,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::Pacific__Samoa => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2445424632, FixedTimespan { utc_offset: -40968, dst_offset: 0, name: "LMT" }),
                    (-1861879032, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "SST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 45432,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Alaska => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3225223727, FixedTimespan { utc_offset: -35976, dst_offset: 0, name: "LMT" }),
                    (-2188951224, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AST" }),
                    (-880200000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AWT" }),
                    (-769395600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "APT" }),
                    (-765378000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AST" }),
                    (-86882400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (-21470400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (-5749200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (9979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (25700400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (41428800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (57754800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (73483200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (89204400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (104932800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (120654000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (126705600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (152103600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (162388800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (183553200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (199281600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (215607600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (230731200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (247057200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (262785600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (278506800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (294235200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (309956400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (325684800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (341406000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (357134400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (372855600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (388584000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (404910000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (420033600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "AHDT" }),
                    (436359600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "YST" }),
                    (439030800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (452084400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (467805600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (483534000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (499255200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (514983600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (530704800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (544618800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (562154400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (576068400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (594208800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (607518000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (625658400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (638967600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (657108000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (671022000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (688557600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (702471600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (720007200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (733921200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (752061600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (765370800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (783511200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (796820400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (814960800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (828874800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (846410400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (860324400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (877860000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (891774000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (909309600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (923223600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (941364000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (954673200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (972813600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (986122800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1004263200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1018177200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1035712800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1049626800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1067162400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1081076400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1099216800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1112526000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1130666400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1143975600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1162116000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1173610800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1194170400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1205060400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1225620000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1236510000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1257069600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1268564400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1289124000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1300014000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1320573600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1331463600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1352023200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1362913200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1383472800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1394362800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1414922400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1425812400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1446372000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1457866800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1478426400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1489316400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1509876000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1520766000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1541325600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1552215600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1572775200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1583665200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1604224800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1615719600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1636279200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1647169200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1667728800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1678618800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1699178400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1710068400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1730628000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1741518000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1762077600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1772967600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1793527200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1805022000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1825581600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1836471600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1857031200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1867921200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1888480800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1899370800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1919930400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1930820400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1951380000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1962874800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (1983434400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (1994324400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2014884000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2025774000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2046333600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2057223600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2077783200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2088673200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2109232800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2120122800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2140682400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2152177200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2172736800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2183626800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2204186400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2215076400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2235636000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2246526000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2267085600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2277975600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2298535200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2309425200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2329984800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2341479600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2362039200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2372929200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2393488800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2404378800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2424938400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2435828400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2456388000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2467278000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2487837600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2499332400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2519892000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2530782000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2551341600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2562231600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2582791200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2593681200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2614240800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2625130800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2645690400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2656580400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2677140000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2688634800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2709194400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2720084400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2740644000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2751534000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2772093600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2782983600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2803543200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2814433200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2834992800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2846487600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2867047200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2877937200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2898496800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2909386800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2929946400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2940836400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2961396000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (2972286000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (2992845600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3003735600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3024295200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3035790000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3056349600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3067239600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3087799200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3098689200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3119248800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3130138800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3150698400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3161588400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3182148000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3193038000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3213597600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3225092400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3245652000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3256542000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3277101600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3287991600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3308551200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3319441200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3340000800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3350890800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3371450400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3382945200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3403504800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3414394800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3434954400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3445844400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3466404000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3477294000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3497853600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3508743600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3529303200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3540193200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3560752800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3572247600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3592807200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3603697200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3624256800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3635146800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3655706400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3666596400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3687156000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3698046000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3718605600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3730100400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3750660000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3761550000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3782109600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3792999600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3813559200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3824449200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3845008800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3855898800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3876458400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3887348400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3907908000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3919402800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3939962400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3950852400, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (3971412000, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (3982302000, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4002861600, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (4013751600, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4034311200, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (4045201200, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4065760800, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                    (4076650800, FixedTimespan { utc_offset: -32400, dst_offset: 3600, name: "AKDT" }),
                    (4097210400, FixedTimespan { utc_offset: -32400, dst_offset: 0, name: "AKST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 50424,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Aleutian => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-3225223727, FixedTimespan { utc_offset: -42398, dst_offset: 0, name: "LMT" }),
                    (-2188944802, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "NST" }),
                    (-880196400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "NWT" }),
                    (-769395600, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "NPT" }),
                    (-765374400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "NST" }),
                    (-86878800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (-21466800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (-5745600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (9982800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (25704000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (41432400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (57758400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (73486800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (89208000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (104936400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (120657600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (126709200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (152107200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (162392400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (183556800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (199285200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (215611200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (230734800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (247060800, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (262789200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (278510400, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (294238800, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (309960000, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (325688400, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (341409600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (357138000, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (372859200, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (388587600, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (404913600, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "BST" }),
                    (420037200, FixedTimespan { utc_offset: -39600, dst_offset: 3600, name: "BDT" }),
                    (436363200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "AHST" }),
                    (439034400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (452088000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (467809200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (483537600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (499258800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (514987200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (530708400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (544622400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (562158000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (576072000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (594212400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (607521600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (625662000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (638971200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (657111600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (671025600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (688561200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (702475200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (720010800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (733924800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (752065200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (765374400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (783514800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (796824000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (814964400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (828878400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (846414000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (860328000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (877863600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (891777600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (909313200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (923227200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (941367600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (954676800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (972817200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (986126400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1004266800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1018180800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1035716400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1049630400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1067166000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1081080000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1099220400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1112529600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1130670000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1143979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1162119600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1173614400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1194174000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1205064000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1225623600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1236513600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1257073200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1268568000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1289127600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1300017600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1320577200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1331467200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1352026800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1362916800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1383476400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1394366400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1414926000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1425816000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1446375600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1457870400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1478430000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1489320000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1509879600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1520769600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1541329200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1552219200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1572778800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1583668800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1604228400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1615723200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1636282800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1647172800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1667732400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1678622400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1699182000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1710072000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1730631600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1741521600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1762081200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1772971200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1793530800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1805025600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1825585200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1836475200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1857034800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1867924800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1888484400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1899374400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1919934000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1930824000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1951383600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1962878400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (1983438000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (1994328000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2014887600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2025777600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2046337200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2057227200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2077786800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2088676800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2109236400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2120126400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2140686000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2152180800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2172740400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2183630400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2204190000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2215080000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2235639600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2246529600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2267089200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2277979200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2298538800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2309428800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2329988400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2341483200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2362042800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2372932800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2393492400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2404382400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2424942000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2435832000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2456391600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2467281600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2487841200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2499336000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2519895600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2530785600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2551345200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2562235200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2582794800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2593684800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2614244400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2625134400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2645694000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2656584000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2677143600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2688638400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2709198000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2720088000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2740647600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2751537600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2772097200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2782987200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2803546800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2814436800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2834996400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2846491200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2867050800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2877940800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2898500400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2909390400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2929950000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2940840000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2961399600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (2972289600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (2992849200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3003739200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3024298800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3035793600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3056353200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3067243200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3087802800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3098692800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3119252400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3130142400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3150702000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3161592000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3182151600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3193041600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3213601200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3225096000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3245655600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3256545600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3277105200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3287995200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3308554800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3319444800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3340004400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3350894400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3371454000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3382948800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3403508400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3414398400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3434958000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3445848000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3466407600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3477297600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3497857200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3508747200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3529306800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3540196800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3560756400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3572251200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3592810800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3603700800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3624260400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3635150400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3655710000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3666600000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3687159600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3698049600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3718609200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3730104000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3750663600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3761553600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3782113200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3793003200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3813562800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3824452800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3845012400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3855902400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3876462000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3887352000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3907911600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3919406400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3939966000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3950856000, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (3971415600, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (3982305600, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4002865200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4013755200, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4034314800, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4045204800, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4065764400, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                    (4076654400, FixedTimespan { utc_offset: -36000, dst_offset: 3600, name: "HDT" }),
                    (4097214000, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 44002,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Arizona => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717643600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1633273200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1615132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1601823600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1583683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-880210800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-820519140, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-812653140, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-796845540, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-84380400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-68659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -26898,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Central => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1563724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1551632400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1538928000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1520182800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1504454400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1491757200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1473004800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1459702800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1441555200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1428253200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1410105600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1396803600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1378656000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1365354000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1347206400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1333904400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1315152000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1301850000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1283702400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1270400400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1252252800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1238950800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1220803200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1207501200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1189353600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1176051600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1157299200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1144602000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1125849600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1112547600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1094400000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1081098000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1067788800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1045414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1031500800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1018198800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1000051200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-986749200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-967996800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-955299600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-936547200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-923245200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-905097600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-891795600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-447267600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-415818000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-384368400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-352918800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-321469200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-289414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-226515600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-210787200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-179337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-163616400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-147888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-131562000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-116438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-100112400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-84384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-5763600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (9964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (25686000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (41414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (57740400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (73468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (89190000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (104918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (120639600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (126691200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (162374400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (183538800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (199267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (215593200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (230716800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (247042800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (262771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (278492400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (294220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (309942000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (325670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (341391600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (357120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (372841200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (388569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (404895600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (420019200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (436345200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (452073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (467794800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (483523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (499244400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (514972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (530694000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (544608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (562143600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (576057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (594198000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (607507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (625647600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (638956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (657097200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (671011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (688546800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (702460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (719996400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (733910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (752050800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (765360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (783500400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (796809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (814950000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (828864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (846399600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (860313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (877849200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (891763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (909298800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (923212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (941353200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (954662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (972802800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (986112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1004252400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1018166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1035702000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1049616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1067151600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1081065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1099206000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1112515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1130655600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1143964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1205049600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1225609200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1236499200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1257058800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1268553600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1289113200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1300003200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1320562800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1331452800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1352012400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1362902400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1383462000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1394352000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1414911600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1425801600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1446361200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1457856000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1478415600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1489305600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1509865200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1520755200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1541314800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1552204800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1572764400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1583654400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1604214000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1615708800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1636268400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1647158400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1667718000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1678608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1699167600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1710057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1730617200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1741507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1762066800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1772956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1793516400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1805011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1825570800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1836460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1857020400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1867910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1888470000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1899360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1919919600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1930809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1951369200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1962864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1983423600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1994313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2014873200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2025763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2046322800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2057212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2077772400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2088662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2109222000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2120112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2140671600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2152166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2172726000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2183616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2204175600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2215065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2235625200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2246515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2267074800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2277964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2298524400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2309414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2329974000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2341468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2362028400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2372918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2393478000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2404368000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2424927600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2435817600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2456377200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2467267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2487826800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2499321600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2519881200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2530771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2551330800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2562220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2582780400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2593670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2614230000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2625120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2645679600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2656569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2677129200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2688624000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2709183600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2720073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2740633200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2751523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2772082800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2782972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2803532400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2814422400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2834982000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2846476800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2867036400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2877926400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2898486000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2909376000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2929935600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2940825600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2961385200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2972275200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2992834800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3003724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3024284400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3035779200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3056338800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3067228800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3087788400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3098678400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3119238000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3130128000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3150687600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3161577600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3182137200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3193027200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3213586800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3225081600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3245641200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3256531200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3277090800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3287980800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3308540400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3319430400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3339990000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3350880000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3371439600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3382934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3403494000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3414384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3434943600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3445833600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3466393200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3477283200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3497842800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3508732800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3529292400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3540182400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3560742000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3572236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3592796400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3603686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3624246000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3635136000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3655695600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3666585600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3687145200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3698035200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3718594800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3730089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3750649200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3761539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3782098800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3792988800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3813548400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3824438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3844998000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3855888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3876447600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3887337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3907897200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3919392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3939951600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3950841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3971401200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3982291200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4002850800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4013740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4034300400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4045190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4065750000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4076640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4097199600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -21036,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__EastIndiana => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-900259200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-891795600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-747244800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-733942800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20678,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Eastern => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717650800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1633280400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1615140000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1601830800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1583690400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1570381200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1551636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1536512400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1523210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1504458000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1491760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1473008400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1459706400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1441558800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1428256800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1410109200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1396807200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1378659600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1365357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1347210000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1333908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1315155600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1301853600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1283706000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1270404000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1252256400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1238954400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1220806800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1207504800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1189357200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1176055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1157302800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1144605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1125853200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1112551200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1094403600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1081101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1062954000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1049652000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1031504400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-1018202400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-1000054800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-986752800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-968000400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-955303200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-936550800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-923248800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-905101200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-891799200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-880218000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EWT" }),
                    (-769395600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EPT" }),
                    (-765396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-747248400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-733946400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-715798800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-702496800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-684349200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-671047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-652899600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-639597600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-620845200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-608148000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-589395600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-576093600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-557946000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-544644000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-526496400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-513194400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-495046800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-481744800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-463597200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-447271200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-431542800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-415821600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-400093200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-384372000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-368643600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-352922400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-337194000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-321472800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-305744400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-289418400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-273690000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-257968800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-242240400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-226519200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-210790800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-195069600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-179341200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-163620000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-147891600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-131565600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-116442000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-100116000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-84387600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-68666400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-52938000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-37216800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-21488400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-5767200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (9961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (25682400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (41410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (57736800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (73465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (89186400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (104914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (120636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (126687600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (152085600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (162370800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (183535200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (199263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (215589600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (230713200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (247039200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (262767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (278488800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (294217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (309938400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (325666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (341388000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (357116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (372837600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (388566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (404892000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (420015600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (436341600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (452070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (467791200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (483519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (499240800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (514969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (530690400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (544604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (562140000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (576054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (594194400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (607503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (625644000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (638953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (657093600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (671007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (688543200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (702457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (719992800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (733906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (752047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (765356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (783496800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (796806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (814946400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (828860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (846396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (860310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (877845600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (891759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (909295200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (923209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (941349600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (954658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (972799200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (986108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1004248800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1018162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1035698400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1049612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1067148000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1081062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1099202400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1112511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1130652000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -17762,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Hawaii => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2334101314, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-1157283000, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HDT" }),
                    (-1155436200, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-880198200, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HWT" }),
                    (-769395600, FixedTimespan { utc_offset: -37800, dst_offset: 3600, name: "HPT" }),
                    (-765376200, FixedTimespan { utc_offset: -37800, dst_offset: 0, name: "HST" }),
                    (-712150200, FixedTimespan { utc_offset: -36000, dst_offset: 0, name: "HST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -37886,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__IndianaStarke => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717647200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1633276800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1615136400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1601827200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-1583686800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-880214400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CWT" }),
                    (-769395600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CPT" }),
                    (-765392400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-715795200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-702493200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-684345600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-671043600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-652896000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-639594000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-620841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-608144400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-589392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-576090000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-557942400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-544640400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-526492800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-513190800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-495043200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-481741200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-463593600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-447267600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-431539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-415818000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-400089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-386787600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-368640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-355338000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-337190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-321469200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-305740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-289414800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-273686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-257965200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-242236800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-195066000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-84384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-68662800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-52934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-37213200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-21484800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (-5763600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (9964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (25686000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (41414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (57740400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (73468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (89190000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (104918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (120639600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (126691200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (152089200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (162374400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (183538800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (199267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (215593200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (230716800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (247042800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (262771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (278492400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (294220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (309942000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (325670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (341391600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (357120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (372841200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (388569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (404895600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (420019200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (436345200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (452073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (467794800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (483523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (499244400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (514972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (530694000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (544608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (562143600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (576057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (594198000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (607507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (625647600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (638956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (657097200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (671011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (688546800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1162105200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1173600000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1194159600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1205049600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1225609200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1236499200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1257058800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1268553600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1289113200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1300003200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1320562800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1331452800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1352012400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1362902400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1383462000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1394352000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1414911600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1425801600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1446361200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1457856000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1478415600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1489305600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1509865200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1520755200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1541314800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1552204800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1572764400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1583654400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1604214000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1615708800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1636268400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1647158400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1667718000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1678608000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1699167600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1710057600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1730617200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1741507200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1762066800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1772956800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1793516400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1805011200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1825570800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1836460800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1857020400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1867910400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1888470000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1899360000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1919919600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1930809600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1951369200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1962864000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (1983423600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (1994313600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2014873200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2025763200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2046322800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2057212800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2077772400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2088662400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2109222000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2120112000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2140671600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2152166400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2172726000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2183616000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2204175600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2215065600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2235625200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2246515200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2267074800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2277964800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2298524400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2309414400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2329974000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2341468800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2362028400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2372918400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2393478000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2404368000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2424927600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2435817600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2456377200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2467267200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2487826800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2499321600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2519881200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2530771200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2551330800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2562220800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2582780400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2593670400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2614230000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2625120000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2645679600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2656569600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2677129200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2688624000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2709183600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2720073600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2740633200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2751523200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2772082800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2782972800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2803532400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2814422400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2834982000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2846476800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2867036400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2877926400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2898486000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2909376000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2929935600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2940825600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2961385200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (2972275200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (2992834800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3003724800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3024284400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3035779200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3056338800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3067228800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3087788400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3098678400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3119238000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3130128000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3150687600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3161577600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3182137200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3193027200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3213586800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3225081600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3245641200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3256531200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3277090800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3287980800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3308540400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3319430400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3339990000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3350880000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3371439600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3382934400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3403494000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3414384000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3434943600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3445833600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3466393200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3477283200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3497842800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3508732800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3529292400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3540182400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3560742000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3572236800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3592796400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3603686400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3624246000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3635136000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3655695600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3666585600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3687145200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3698035200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3718594800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3730089600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3750649200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3761539200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3782098800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3792988800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3813548400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3824438400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3844998000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3855888000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3876447600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3887337600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3907897200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3919392000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3939951600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3950841600, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (3971401200, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (3982291200, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4002850800, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4013740800, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4034300400, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4045190400, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4065750000, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (4076640000, FixedTimespan { utc_offset: -21600, dst_offset: 3600, name: "CDT" }),
                    (4097199600, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -20790,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Michigan => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2051202469, FixedTimespan { utc_offset: -21600, dst_offset: 0, name: "CST" }),
                    (-1724083200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-880218000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EWT" }),
                    (-769395600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EPT" }),
                    (-765396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-684349200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-671047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-80506740, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-68666400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (-52938000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (-37216800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (104914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (120636000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (126687600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (152085600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (167814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (183535200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (199263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (215589600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (230713200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (247039200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (262767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (278488800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (294217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (309938400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (325666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (341388000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (357116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (372837600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (388566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (404892000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (420015600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (436341600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (452070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (467791200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (483519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (499240800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (514969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (530690400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (544604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (562140000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (576054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (594194400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (607503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (625644000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (638953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (657093600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (671007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (688543200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (702457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (719992800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (733906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (752047200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (765356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (783496800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (796806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (814946400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (828860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (846396000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (860310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (877845600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (891759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (909295200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (923209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (941349600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (954658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (972799200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (986108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1004248800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1018162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1035698400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1049612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1067148000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1081062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1099202400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1112511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1130652000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1143961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1162101600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1173596400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1194156000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1205046000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1225605600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1236495600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1257055200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1268550000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1289109600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1299999600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1320559200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1331449200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1352008800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1362898800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1383458400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1394348400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1414908000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1425798000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1446357600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1457852400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1478412000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1489302000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1509861600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1520751600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1541311200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1552201200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1572760800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1583650800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1604210400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1615705200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1636264800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1647154800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1667714400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1678604400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1699164000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1710054000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1730613600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1741503600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1762063200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1772953200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1793512800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1805007600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1825567200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1836457200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1857016800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1867906800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1888466400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1899356400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1919916000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1930806000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1951365600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1962860400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (1983420000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (1994310000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2014869600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2025759600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2046319200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2057209200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2077768800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2088658800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2109218400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2120108400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2140668000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2152162800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2172722400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2183612400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2204172000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2215062000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2235621600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2246511600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2267071200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2277961200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2298520800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2309410800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2329970400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2341465200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2362024800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2372914800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2393474400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2404364400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2424924000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2435814000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2456373600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2467263600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2487823200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2499318000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2519877600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2530767600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2551327200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2562217200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2582776800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2593666800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2614226400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2625116400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2645676000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2656566000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2677125600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2688620400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2709180000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2720070000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2740629600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2751519600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2772079200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2782969200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2803528800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2814418800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2834978400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2846473200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2867032800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2877922800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2898482400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2909372400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2929932000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2940822000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2961381600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (2972271600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (2992831200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3003721200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3024280800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3035775600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3056335200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3067225200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3087784800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3098674800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3119234400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3130124400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3150684000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3161574000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3182133600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3193023600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3213583200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3225078000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3245637600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3256527600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3277087200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3287977200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3308536800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3319426800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3339986400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3350876400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3371436000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3382930800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3403490400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3414380400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3434940000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3445830000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3466389600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3477279600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3497839200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3508729200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3529288800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3540178800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3560738400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3572233200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3592792800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3603682800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3624242400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3635132400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3655692000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3666582000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3687141600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3698031600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3718591200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3730086000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3750645600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3761535600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3782095200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3792985200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3813544800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3824434800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3844994400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3855884400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3876444000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3887334000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3907893600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3919388400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3939948000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3950838000, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (3971397600, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (3982287600, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4002847200, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4013737200, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4034296800, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4045186800, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4065746400, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                    (4076636400, FixedTimespan { utc_offset: -18000, dst_offset: 3600, name: "EDT" }),
                    (4097196000, FixedTimespan { utc_offset: -18000, dst_offset: 0, name: "EST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -19931,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Mountain => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717643600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1633273200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1615132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1601823600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1583683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1570374000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1551628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-1538924400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-1534089600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-880210800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MWT" }),
                    (-769395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MPT" }),
                    (-765388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-147884400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-131558400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-116434800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-100108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-84380400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-68659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-52930800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-37209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (-21481200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (-5760000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (9968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (25689600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (41418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (57744000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (73472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (89193600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (104922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (120643200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (126694800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (152092800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (162378000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (183542400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (199270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (215596800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (230720400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (247046400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (262774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (278496000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (294224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (309945600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (325674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (341395200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (357123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (372844800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (388573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (404899200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (420022800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (436348800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (452077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (467798400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (483526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (499248000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (514976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (530697600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (544611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (562147200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (576061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (594201600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (607510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (625651200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (638960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (657100800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (671014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (688550400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (702464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (720000000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (733914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (752054400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (765363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (783504000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (796813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (814953600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (828867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (846403200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (860317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (877852800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (891766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (909302400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (923216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (941356800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (954666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (972806400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (986115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1004256000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1018170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1035705600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1049619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1067155200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1081069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1099209600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1112518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1130659200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1143968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1162108800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1173603600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1194163200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1205053200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1225612800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1236502800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1257062400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1268557200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1289116800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1300006800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1320566400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1331456400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1352016000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1362906000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1383465600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1394355600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1414915200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1425805200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1446364800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1457859600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1478419200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1489309200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1509868800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1520758800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1541318400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1552208400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1572768000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1583658000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1604217600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1615712400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1636272000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1647162000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1667721600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1678611600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1699171200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1710061200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1730620800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1741510800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1762070400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1772960400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1793520000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1805014800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1825574400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1836464400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1857024000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1867914000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1888473600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1899363600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1919923200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1930813200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1951372800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1962867600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (1983427200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (1994317200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2014876800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2025766800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2046326400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2057216400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2077776000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2088666000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2109225600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2120115600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2140675200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2152170000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2172729600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2183619600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2204179200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2215069200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2235628800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2246518800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2267078400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2277968400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2298528000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2309418000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2329977600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2341472400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2362032000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2372922000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2393481600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2404371600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2424931200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2435821200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2456380800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2467270800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2487830400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2499325200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2519884800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2530774800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2551334400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2562224400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2582784000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2593674000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2614233600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2625123600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2645683200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2656573200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2677132800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2688627600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2709187200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2720077200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2740636800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2751526800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2772086400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2782976400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2803536000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2814426000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2834985600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2846480400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2867040000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2877930000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2898489600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2909379600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2929939200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2940829200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2961388800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (2972278800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (2992838400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3003728400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3024288000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3035782800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3056342400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3067232400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3087792000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3098682000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3119241600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3130131600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3150691200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3161581200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3182140800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3193030800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3213590400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3225085200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3245644800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3256534800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3277094400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3287984400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3308544000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3319434000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3339993600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3350883600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3371443200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3382938000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3403497600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3414387600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3434947200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3445837200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3466396800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3477286800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3497846400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3508736400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3529296000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3540186000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3560745600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3572240400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3592800000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3603690000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3624249600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3635139600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3655699200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3666589200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3687148800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3698038800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3718598400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3730093200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3750652800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3761542800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3782102400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3792992400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3813552000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3824442000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3845001600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3855891600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3876451200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3887341200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3907900800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3919395600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3939955200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3950845200, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (3971404800, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (3982294800, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4002854400, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4013744400, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4034304000, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4045194000, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4065753600, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                    (4076643600, FixedTimespan { utc_offset: -25200, dst_offset: 3600, name: "MDT" }),
                    (4097203200, FixedTimespan { utc_offset: -25200, dst_offset: 0, name: "MST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -25196,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Pacific => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717640000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-1633269600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-1615129200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-1601820000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-1583679600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-880207200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PWT" }),
                    (-769395600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PPT" }),
                    (-765385200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-687967140, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-662655600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-620838000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-608137200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-589388400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-576082800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-557938800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-544633200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-526489200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-513183600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-495039600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-481734000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-463590000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-450284400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-431535600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-418230000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-400086000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-386780400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-368636400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-355330800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-337186800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-323881200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-305737200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-292431600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-273682800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-260982000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-242233200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-226508400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-210783600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-195058800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-179334000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-163609200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-147884400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-131554800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-116434800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-100105200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-84376800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-68655600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-52927200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-37206000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-21477600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-5756400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (9972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (25693200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (41421600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (57747600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (73476000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (89197200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (104925600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (120646800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (126698400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (152096400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (162381600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (183546000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (199274400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (215600400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (230724000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (247050000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (262778400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (278499600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (294228000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (309949200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (325677600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (341398800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (357127200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (372848400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (388576800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (404902800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (420026400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (436352400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (452080800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (467802000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (483530400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (499251600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (514980000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (530701200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (544615200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (562150800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (576064800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (594205200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (607514400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (625654800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (638964000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (657104400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (671018400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (688554000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (702468000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (720003600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (733917600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (752058000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (765367200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (783507600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (796816800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (814957200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (828871200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (846406800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (860320800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (877856400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (891770400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (909306000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (923220000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (941360400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (954669600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (972810000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (986119200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1004259600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1018173600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1035709200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1049623200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1067158800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1081072800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1099213200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1112522400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1130662800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1143972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1162112400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1173607200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1194166800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1205056800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1225616400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1236506400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1257066000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1268560800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1289120400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1300010400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1320570000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1331460000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1352019600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1362909600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1383469200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1394359200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1414918800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1425808800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1446368400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1457863200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1478422800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1489312800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1509872400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1520762400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1541322000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1552212000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1572771600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1583661600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1604221200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1615716000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1636275600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1647165600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1667725200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1678615200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1699174800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1710064800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1730624400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1741514400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1762074000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1772964000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1793523600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1805018400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1825578000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1836468000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1857027600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1867917600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1888477200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1899367200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1919926800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1930816800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1951376400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1962871200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1983430800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1994320800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2014880400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2025770400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2046330000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2057220000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2077779600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2088669600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2109229200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2120119200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2140678800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2152173600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2172733200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2183623200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2204182800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2215072800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2235632400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2246522400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2267082000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2277972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2298531600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2309421600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2329981200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2341476000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2362035600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2372925600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2393485200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2404375200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2424934800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2435824800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2456384400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2467274400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2487834000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2499328800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2519888400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2530778400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2551338000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2562228000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2582787600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2593677600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2614237200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2625127200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2645686800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2656576800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2677136400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2688631200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2709190800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2720080800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2740640400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2751530400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2772090000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2782980000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2803539600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2814429600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2834989200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2846484000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2867043600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2877933600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2898493200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2909383200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2929942800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2940832800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2961392400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2972282400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2992842000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3003732000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3024291600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3035786400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3056346000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3067236000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3087795600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3098685600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3119245200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3130135200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3150694800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3161584800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3182144400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3193034400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3213594000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3225088800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3245648400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3256538400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3277098000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3287988000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3308547600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3319437600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3339997200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3350887200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3371446800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3382941600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3403501200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3414391200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3434950800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3445840800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3466400400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3477290400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3497850000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3508740000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3529299600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3540189600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3560749200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3572244000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3592803600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3603693600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3624253200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3635143200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3655702800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3666592800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3687152400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3698042400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3718602000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3730096800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3750656400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3761546400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3782106000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3792996000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3813555600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3824445600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3845005200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3855895200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3876454800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3887344800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3907904400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3919399200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3939958800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3950848800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3971408400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3982298400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4002858000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4013748000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4034307600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4045197600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4065757200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4076647200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4097206800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -28378,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__PacificNew => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2717640000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-1633269600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-1615129200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-1601820000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-1583679600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-880207200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PWT" }),
                    (-769395600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PPT" }),
                    (-765385200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-687967140, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-662655600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-620838000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-608137200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-589388400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-576082800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-557938800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-544633200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-526489200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-513183600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-495039600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-481734000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-463590000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-450284400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-431535600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-418230000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-400086000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-386780400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-368636400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-355330800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-337186800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-323881200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-305737200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-292431600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-273682800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-260982000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-242233200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-226508400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-210783600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-195058800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-179334000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-163609200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-147884400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-131554800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-116434800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-100105200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-84376800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-68655600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-52927200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-37206000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (-21477600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (-5756400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (9972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (25693200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (41421600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (57747600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (73476000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (89197200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (104925600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (120646800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (126698400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (152096400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (162381600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (183546000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (199274400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (215600400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (230724000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (247050000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (262778400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (278499600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (294228000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (309949200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (325677600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (341398800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (357127200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (372848400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (388576800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (404902800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (420026400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (436352400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (452080800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (467802000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (483530400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (499251600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (514980000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (530701200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (544615200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (562150800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (576064800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (594205200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (607514400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (625654800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (638964000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (657104400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (671018400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (688554000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (702468000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (720003600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (733917600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (752058000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (765367200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (783507600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (796816800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (814957200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (828871200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (846406800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (860320800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (877856400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (891770400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (909306000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (923220000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (941360400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (954669600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (972810000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (986119200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1004259600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1018173600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1035709200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1049623200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1067158800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1081072800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1099213200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1112522400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1130662800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1143972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1162112400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1173607200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1194166800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1205056800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1225616400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1236506400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1257066000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1268560800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1289120400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1300010400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1320570000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1331460000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1352019600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1362909600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1383469200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1394359200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1414918800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1425808800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1446368400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1457863200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1478422800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1489312800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1509872400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1520762400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1541322000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1552212000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1572771600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1583661600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1604221200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1615716000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1636275600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1647165600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1667725200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1678615200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1699174800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1710064800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1730624400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1741514400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1762074000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1772964000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1793523600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1805018400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1825578000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1836468000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1857027600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1867917600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1888477200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1899367200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1919926800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1930816800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1951376400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1962871200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (1983430800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (1994320800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2014880400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2025770400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2046330000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2057220000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2077779600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2088669600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2109229200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2120119200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2140678800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2152173600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2172733200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2183623200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2204182800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2215072800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2235632400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2246522400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2267082000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2277972000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2298531600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2309421600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2329981200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2341476000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2362035600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2372925600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2393485200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2404375200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2424934800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2435824800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2456384400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2467274400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2487834000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2499328800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2519888400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2530778400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2551338000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2562228000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2582787600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2593677600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2614237200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2625127200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2645686800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2656576800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2677136400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2688631200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2709190800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2720080800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2740640400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2751530400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2772090000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2782980000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2803539600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2814429600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2834989200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2846484000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2867043600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2877933600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2898493200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2909383200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2929942800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2940832800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2961392400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (2972282400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (2992842000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3003732000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3024291600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3035786400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3056346000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3067236000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3087795600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3098685600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3119245200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3130135200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3150694800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3161584800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3182144400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3193034400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3213594000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3225088800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3245648400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3256538400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3277098000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3287988000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3308547600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3319437600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3339997200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3350887200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3371446800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3382941600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3403501200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3414391200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3434950800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3445840800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3466400400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3477290400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3497850000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3508740000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3529299600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3540189600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3560749200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3572244000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3592803600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3603693600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3624253200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3635143200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3655702800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3666592800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3687152400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3698042400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3718602000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3730096800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3750656400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3761546400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3782106000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3792996000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3813555600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3824445600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3845005200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3855895200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3876454800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3887344800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3907904400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3919399200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3939958800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3950848800, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (3971408400, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (3982298400, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4002858000, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4013748000, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4034307600, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4045197600, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4065757200, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                    (4076647200, FixedTimespan { utc_offset: -28800, dst_offset: 3600, name: "PDT" }),
                    (4097206800, FixedTimespan { utc_offset: -28800, dst_offset: 0, name: "PST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: -28378,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

            Tz::US__Samoa => {
                const REST: &[(i64, FixedTimespan)] = &[
                    (-2445424632, FixedTimespan { utc_offset: -40968, dst_offset: 0, name: "LMT" }),
                    (-1861879032, FixedTimespan { utc_offset: -39600, dst_offset: 0, name: "SST" }),
                ];
                FixedTimespanSet {
                    first: FixedTimespan {
                        utc_offset: 45432,
                        dst_offset: 0,
                        name: "LMT",
                    },
                    rest: REST
                }
            },

         }
    }
}
/// An array of every known variant
///
/// Useful for iterating over known timezones:
///
/// ```
/// use chrono_tz::{TZ_VARIANTS, Tz};
/// assert!(TZ_VARIANTS.iter().any(|v| *v == Tz::UTC));
/// ```
pub static TZ_VARIANTS: [Tz; 40] = [
    Tz::America__Adak,
    Tz::America__Anchorage,
    Tz::America__Atka,
    Tz::America__Chicago,
    Tz::America__Denver,
    Tz::America__Detroit,
    Tz::America__Fort_Wayne,
    Tz::America__Indiana__Indianapolis,
    Tz::America__Indiana__Knox,
    Tz::America__Indiana__Marengo,
    Tz::America__Indiana__Petersburg,
    Tz::America__Indiana__Tell_City,
    Tz::America__Indiana__Vevay,
    Tz::America__Indiana__Vincennes,
    Tz::America__Indiana__Winamac,
    Tz::America__Indianapolis,
    Tz::America__Knox_IN,
    Tz::America__Los_Angeles,
    Tz::America__New_York,
    Tz::America__Phoenix,
    Tz::America__Shiprock,
    Tz::Navajo,
    Tz::Pacific__Honolulu,
    Tz::Pacific__Johnston,
    Tz::Pacific__Midway,
    Tz::Pacific__Pago_Pago,
    Tz::Pacific__Samoa,
    Tz::US__Alaska,
    Tz::US__Aleutian,
    Tz::US__Arizona,
    Tz::US__Central,
    Tz::US__EastIndiana,
    Tz::US__Eastern,
    Tz::US__Hawaii,
    Tz::US__IndianaStarke,
    Tz::US__Michigan,
    Tz::US__Mountain,
    Tz::US__Pacific,
    Tz::US__PacificNew,
    Tz::US__Samoa,
];